import logging
from typing import Tuple, Union

from mongoengine import QuerySet
from spaceone.core.manager import BaseManager

from spaceone.identity.model import Agent


_LOGGER = logging.getLogger(__name__)


class AgentManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.agent_model = Agent

    def create_agent(self, params: dict) -> Agent:
        def _rollback(vo: Agent):
            _LOGGER.info(f"[create_agent._rollback] " f"Delete agent: ({vo.agent_id})")
            agent_vo.delete()

        agent_vo = self.agent_model.create(params)
        self.transaction.add_rollback(_rollback, agent_vo)

        return agent_vo

    def get_agent(
        self,
        service_account_id: str,
        domain_id: str,
        workspace_id: str,
        user_projects: Union[list, None] = None,
    ) -> Agent:
        conditions = {
            "service_account_id": service_account_id,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
        }

        if user_projects:
            conditions["project_id"] = user_projects

        return self.agent_model.get(**conditions)

    def update_agent_by_vo(self, params: dict, agent_vo: Agent) -> Agent:
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_agent_by_vo._rollback] Revert Data : "
                f"{old_data['agent_id']}"
            )
            agent_vo.update(old_data)

        self.transaction.add_rollback(_rollback, agent_vo.to_dict())

        return agent_vo.update(params)

    @staticmethod
    def delete_agent_by_vo(agent_vo: Agent) -> None:
        agent_vo.delete()

    def filter_agents(self, **conditions) -> QuerySet:
        return self.agent_model.filter(**conditions)

    def list_agents(self, query: dict) -> Tuple[list, int]:
        return self.agent_model.query(**query)



from datetime import datetime
from typing import Union, Literal, List
from pydantic import BaseModel

from spaceone.core import utils

__all__ = ["JobResponse", "JobsResponse"]

ResourceGroup = Literal["DOMAIN", "WORKSPACE"]
Status = Literal["PENDING", "IN_PROGRESS", "FAILURE", "SUCCESS", "CANCELED"]


class JobResponse(BaseModel):
    job_id: Union[str, None] = None
    status: Union[Status, None] = None
    options: Union[dict, None] = None
    error_message: Union[str, None] = None
    resource_group: Union[ResourceGroup, None] = None
    trusted_account_id: Union[str, None] = None
    plugin_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None
    finished_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["updated_at"] = utils.datetime_to_iso8601(data.get("updated_at"))
        data["finished_at"] = utils.datetime_to_iso8601(data.get("finished_at"))
        return data


class JobsResponse(BaseModel):
    results: List[JobResponse] = []
    total_count: int



from datetime import datetime

from mongoengine import *
from spaceone.core.error import *
from spaceone.core.model.mongo_model import MongoModel


class Workspace(MongoModel):
    workspace_id = StringField(max_length=40, generate_id="workspace", unique=True)
    name = StringField(max_length=255)
    state = StringField(
        max_length=20, default="ENABLED", choices=["ENABLED", "DISABLED", "DELETED"]
    )
    tags = DictField(default=None)
    created_by = StringField(max_length=255)
    references = ListField(StringField(max_length=255), default=None, null=True)
    is_managed = BooleanField(default=False)

    is_dormant = BooleanField(default=False)
    dormant_ttl = IntField(default=None, required=True)
    service_account_count = IntField(default=None)
    user_count = IntField(default=None)
    cost_info = DictField(default=None)

    trusted_account_id = StringField(max_length=40, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    deleted_at = DateTimeField(default=None, null=True)
    last_synced_at = DateTimeField(default=None, null=True)
    dormant_updated_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "tags",
            "is_managed",
            "is_dormant",
            "dormant_ttl",
            "service_account_count",
            "user_count",
            "cost_info",
            "trusted_account_id",
            "references",
            "deleted_at",
            "last_synced_at",
            "dormant_updated_at",
        ],
        "minimal_fields": [
            "workspace_id",
            "name",
            "state",
            "is_managed",
            "is_dormant",
        ],
        "change_query_keys": {
            "reference_id": "references",
        },
        "ordering": ["name"],
        "indexes": [
            "name",
            "state",
            "domain_id",
            "created_by",
            "is_managed",
            "is_dormant",
        ],
    }

    @queryset_manager
    def objects(doc_cls, query_set: QuerySet):
        return query_set.filter(state__ne="DELETED")

    @classmethod
    def create(cls, data):
        workspace_vos = cls.filter(name=data["name"], domain_id=data["domain_id"])
        if workspace_vos.count() > 0:
            raise ERROR_NOT_UNIQUE(key="name", value=data["name"])

        return super().create(data)

    def delete(self):
        self.update({"state": "DELETED", "deleted_at": datetime.utcnow()})



import logging
from typing import Tuple
from mongoengine import QuerySet

from spaceone.core import cache
from spaceone.core.manager import BaseManager
from spaceone.identity.model.domain.database import Domain

_LOGGER = logging.getLogger(__name__)


class DomainManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_model = Domain

    def create_domain(self, params: dict) -> Domain:
        def _rollback(vo: Domain):
            _LOGGER.info(
                f"[create_domain._rollback] Delete domain: {vo.name} ({vo.domain_id})"
            )
            vo.delete()

        domain_vo = self.domain_model.create(params)
        self.transaction.add_rollback(_rollback, domain_vo)

        return domain_vo

    def update_domain_by_vo(self, params: dict, domain_vo: Domain) -> Domain:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_domain._rollback] Revert Data: {old_data["name"]} ({old_data["domain_id"]})'
            )
            domain_vo.update(old_data)

        self.transaction.add_rollback(_rollback, domain_vo.to_dict())

        return domain_vo.update(params)

    @staticmethod
    def delete_domain_by_vo(domain_vo: Domain) -> None:
        domain_vo.delete()
        cache.delete_pattern(f"identity:domain-state:{domain_vo.domain_id}")

    def enable_domain(self, domain_vo: Domain) -> Domain:
        self.update_domain_by_vo({"state": "ENABLED"}, domain_vo)
        cache.delete_pattern(f"identity:domain-state:{domain_vo.domain_id}")

        return domain_vo

    def disable_domain(self, domain_vo: Domain) -> Domain:
        self.update_domain_by_vo({"state": "DISABLED"}, domain_vo)
        cache.delete_pattern(f"identity:domain-state:{domain_vo.domain_id}")

        return domain_vo

    def get_domain(self, domain_id: str) -> Domain:
        return self.domain_model.get(domain_id=domain_id)

    def get_domain_by_name(self, name: str) -> Domain:
        return self.domain_model.get(name=name)

    def filter_domains(self, **conditions) -> QuerySet:
        return self.domain_model.filter(**conditions)

    def list_domains(self, query: dict) -> Tuple[QuerySet, int]:
        return self.domain_model.query(**query)

    def stat_domains(self, query: dict) -> dict:
        return self.domain_model.stat(**query)



import logging
from typing import Union

from spaceone.core.error import (ERROR_INVALID_PARAMETER, ERROR_NOT_FOUND,
                                 ERROR_PERMISSION_DENIED)
from spaceone.core.service import (BaseService, authentication_handler,
                                   authorization_handler, event_handler,
                                   mutation_handler, transaction)
from spaceone.core.service.utils import (append_keyword_filter,
                                         append_query_filter, convert_model)

from spaceone.identity.error.error_role import (ERROR_NOT_ALLOWED_ROLE_TYPE,
                                                ERROR_NOT_ALLOWED_USER_STATE)
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.workspace_group_manager import \
    WorkspaceGroupManager
from spaceone.identity.manager.workspace_group_user_manager import \
    WorkspaceGroupUserManager
from spaceone.identity.model.workspace_group.response import \
    WorkspaceGroupResponse
from spaceone.identity.model.workspace_group_user.request import (
    WorkspaceGroupUserAddRequest, WorkspaceGroupUserFindRequest,
    WorkspaceGroupUserGetRequest, WorkspaceGroupUserRemoveRequest,
    WorkspaceGroupUserSearchQueryRequest, WorkspaceGroupUserStatQueryRequest,
    WorkspaceGroupUserUpdateRoleRequest)
from spaceone.identity.model.workspace_group_user.response import (
    WorkspaceGroupUserResponse, WorkspaceGroupUsersResponse,
    WorkspaceGroupUsersSummaryResponse)
from spaceone.identity.service.role_binding_service import RoleBindingService

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class WorkspaceGroupUserService(BaseService):
    resource = "WorkspaceGroupUser"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_mgr = UserManager()
        self.workspace_group_mgr = WorkspaceGroupManager()
        self.workspace_group_user_mgr = WorkspaceGroupUserManager()
        self.role_mgr = RoleManager()
        self.rb_svc = RoleBindingService()
        self.rb_mgr = RoleBindingManager()

    @transaction(permission="identity:WorkspaceGroupUser:write", role_types=["USER"])
    @convert_model
    def add(
        self, params: WorkspaceGroupUserAddRequest
    ) -> Union[WorkspaceGroupUserResponse, dict]:
        """Add workspace group user
        Args:
            params (WorkspaceGroupUserAddRequest): {
                'workspace_group_id': 'str',     # required
                'users': [
                    {
                        'user_id': 'str',        # required
                        'role_id': 'str',        # required
                    }
                ],
                'user_id': 'str',                # injected from auth (required)
                'domain_id': 'str',              # injected from auth (required)
            }
        Returns:
           WorkspaceGroupUserResponse:
        """
        user_id = self.transaction.get_meta("authorization.user_id")

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        user_role_type = ""
        for user in workspace_group_vo.users:
            if user["user_id"] == user_id:
                user_role_type = user["role_type"]

        if user_role_type == "WORKSPACE_MEMBER":
            _LOGGER.error(
                f"User ID {user_id} does not have permission to add users to workspace group."
            )
            raise ERROR_PERMISSION_DENIED()

        new_users = list(set([user_info["user_id"] for user_info in params.users]))
        old_users = []
        if workspace_group_vo.users:
            old_users = list(
                set([user_info["user_id"] for user_info in workspace_group_vo.users])
            )

        if set(new_users) & set(old_users):
            _LOGGER.error(
                f"Users {new_users} is already added to the workspace group or not registered."
            )
            raise ERROR_INVALID_PARAMETER(
                key="users",
                reason=f"User {new_users} is already added to the workspace group or not registered.",
            )

        user_vos = self.user_mgr.filter_users(
            domain_id=params.domain_id, user_id=new_users
        )

        if not user_vos.count() == len(new_users):
            raise ERROR_NOT_FOUND(key="user_id", value=new_users)

        if user_id not in old_users:
            _LOGGER.error(f"User ID {user_id} is not in workspace group.")
            raise ERROR_PERMISSION_DENIED()

        users = params.users or []
        workspaces = workspace_group_vo.workspaces or []

        role_ids = list(set([user["role_id"] for user in users]))
        role_vos = self.role_mgr.filter_roles(
            role_id=role_ids,
            domain_id=params.domain_id,
            role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
        )

        role_map = {role_vo.role_id: role_vo.role_type for role_vo in role_vos}

        user_info = {user_vo["user_id"]: user_vo for user_vo in user_vos}
        for user in users:
            user_id = user["user_id"]
            if user_id in user_info:
                user["user_name"] = user_info[user_id]["name"]
                user["state"] = user_info[user_id]["state"]

        new_users = []
        for user in users:
            role_type = role_map[user["role_id"]]

            for workspace_id in workspaces:
                role_binding_params = {
                    "user_id": user["user_id"],
                    "role_id": user["role_id"],
                    "role_type": role_type,
                    "resource_group": "WORKSPACE",
                    "domain_id": params.domain_id,
                    "workspace_group_id": params.workspace_group_id,
                    "workspace_id": workspace_id,
                }
                self.rb_svc.create_role_binding(role_binding_params)

            new_users.append(
                {
                    "user_id": user["user_id"],
                    "role_id": user["role_id"],
                    "role_type": role_type,
                    "user_name": user["user_name"],
                    "state": user["state"],
                }
            )

        old_users = workspace_group_vo.users or []

        params.users = old_users + new_users

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupUserResponse(**workspace_group_vo.to_dict())

    @transaction(permission="identity:WorkspaceGroupUser:write", role_types=["USER"])
    @convert_model
    def remove(
        self, params: WorkspaceGroupUserRemoveRequest
    ) -> Union[WorkspaceGroupUserResponse, dict]:
        """Remove workspace group user
        Args:
            params (WorkspaceGroupUserRemoveRequest): {
                'workspace_group_id': 'str',        # required
                'users': [
                    {
                        'user_id': 'str',           # required
                    }
                ],
                'user_id': 'str',                   # injected from auth (required)
                'domain_id': 'str',                 # injected from auth (required)
            }
        Returns:
            WorkspaceGroupUserResponse:
        """
        user_id = self.transaction.get_meta("authorization.user_id")

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        user_role_type = ""
        for user in workspace_group_vo.users:
            if user["user_id"] == user_id:
                user_role_type = user["role_type"]

        if user_role_type == "WORKSPACE_MEMBER":
            _LOGGER.error(
                f"User ID {user_id} does not have permission to add users to workspace group."
            )
            raise ERROR_PERMISSION_DENIED()

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id,
            params.domain_id,
            user_id=user_id,
        )
        if workspace_group_vo:
            if not self.workspace_group_mgr.check_user_id_in_users(
                user_id, workspace_group_vo
            ):
                _LOGGER.error(f"User ID {user_id} is not in workspace group.")
                ERROR_NOT_FOUND(
                    key="workspace_group_id", value=params.workspace_group_id
                )
        else:
            ERROR_NOT_FOUND(key="workspace_group_id", value=params.workspace_group_id)

        workspace_group_user_ids = [
            user["user_id"] for user in workspace_group_vo.users
        ]
        params_user_ids = list(set([user["user_id"] for user in params.users]))

        for params_user_id in params_user_ids:
            if params_user_id not in workspace_group_user_ids:
                raise ERROR_NOT_FOUND(key="params_user_id", value=params_user_id)

        workspace_group_users = [users for users in workspace_group_vo["users"]]
        role_binding_vos = self.rb_mgr.filter_role_bindings(
            user_id=params_user_ids,
            workspace_group_id=params.workspace_group_id,
            domain_id=params.domain_id,
        )
        role_binding_vos.delete()

        params.users = []
        for user in workspace_group_users:
            if user["user_id"] not in params_user_ids:
                params.users.append(user)

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupUserResponse(**workspace_group_vo.to_dict())

    @transaction(permission="identity:WorkspaceGroupUser:write", role_types=["USER"])
    @convert_model
    def update_role(
        self, params: WorkspaceGroupUserUpdateRoleRequest
    ) -> Union[WorkspaceGroupUserResponse, dict]:
        """Update workspace group user role
        Args:
            params (WorkspaceGroupUserUpdateRoleRequest): {
                'workspace_group_id': 'str',            # required
                'role_id': 'str',                       # required
                'target_user_id': 'str',                # required
                'user_id': 'str',                       # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
            }
        Returns:
            WorkspaceGroupUserResponse:
        """
        user_id = self.transaction.get_meta("authorization.user_id")

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        user_role_type = ""
        for user in workspace_group_vo.users:
            if user["user_id"] == user_id:
                user_role_type = user["role_type"]

        if user_role_type == "WORKSPACE_MEMBER":
            _LOGGER.error(
                f"User ID {user_id} does not have permission to add users to workspace group."
            )
            raise ERROR_PERMISSION_DENIED()

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )
        user_vo = self.user_mgr.get_user(params.target_user_id, params.domain_id)
        if user_vo.state in ["DISABLED", "DELETED"]:
            _LOGGER.error(f"User ID {user_vo.user_id}'s state is {user_vo.state}.")
            raise ERROR_NOT_ALLOWED_USER_STATE(
                user_id=user_vo.user_id, state=user_vo.state
            )

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        if role_vo.role_type not in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            raise ERROR_NOT_ALLOWED_ROLE_TYPE()

        role_binding_vos = self.rb_mgr.filter_role_bindings(
            user_id=user_vo.user_id,
            workspace_group_id=workspace_group_vo.workspace_group_id,
            domain_id=params.domain_id,
        )

        for role_binding_vo in role_binding_vos:
            role_binding_vo.update(
                {"role_id": params.role_id, "role_type": role_vo.role_type}
            )

        update_workspace_group_params = {"users": workspace_group_vo.users or []}
        for user_info in update_workspace_group_params.get("users", []):
            if user_info["user_id"] == user_vo.user_id:
                user_info["role_id"] = params.role_id
                user_info["role_type"] = role_vo.role_type
                user_info["user_name"] = user_vo.name
                user_info["state"] = user_vo.state
                break

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            update_workspace_group_params, workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(permission="identity:WorkspaceGroupUser:read", role_types=["USER"])
    @convert_model
    def find(
        self, params: WorkspaceGroupUserFindRequest
    ) -> Union[WorkspaceGroupUsersSummaryResponse, dict]:
        """Find users in the domain except users in its workspace_group
        Args:
            params (WorkspaceGroupUserFindRequest): {
                'workspace_group_id': 'str',      # required
                'keyword': 'str',
                'state': 'State',
                'page': 'dict',
                'user_id': 'str'                  # injected from auth (required)
                'domain_id': 'str',               # injected from auth (required)
            }
        Returns:
            WorkspaceGroupUsersSummaryResponse:
        """
        return self._find(params)

    @transaction(permission="identity:WorkspaceGroupUser:read", role_types=["USER"])
    @convert_model
    def get(
        self, params: WorkspaceGroupUserGetRequest
    ) -> Union[WorkspaceGroupUserResponse, dict]:
        """Get workspace groups
        Args:
            params (WorkspaceGroupUserGetRequest): {
                'workspace_group_id': 'str',     # required
                'user_id': 'str'                 # injected from auth (required)
                'domain_id': 'str',              # injected from auth (required)
            }
        Returns:
            WorkspaceGroupUserResponse:
        """
        user_info = self.workspace_group_user_mgr.get_workspace_group_user(
            params.user_id, params.workspace_group_id, params.domain_id
        )
        return WorkspaceGroupUserResponse(**user_info)

    @transaction(permission="identity:WorkspaceGroupUser.read", role_types=["USER"])
    @append_query_filter(["user_id", "workspace_group_id", "name", "domain_id"])
    @append_keyword_filter(["user_id", "workspace_group_id", "name"])
    @convert_model
    def list(
        self, params: WorkspaceGroupUserSearchQueryRequest
    ) -> Union[WorkspaceGroupUsersResponse, dict]:
        """List workspace group users
        Args:
            params (WorkspaceGroupUserSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'workspace_group_id': 'str',
                'name': 'str',
                'created_by': 'str',
                'updated_by': 'str',
                'user_id': 'str'                         # injected from auth (required)
                'domain_id': 'str',                      # injected from auth (required)
            }
        Returns:
            WorkspaceGroupUsersResponse:
        """
        query = params.query or {}
        users_info, total_count = (
            self.workspace_group_user_mgr.list_workspace_group_users(
                query,
                params.user_id,
                params.domain_id,
            )
        )

        return WorkspaceGroupUsersResponse(results=users_info, total_count=total_count)

    @transaction(permission="identity:WorkspaceGroup.read", role_types=["USER"])
    @append_query_filter(["user_id", "workspace_group_id", "domain_id"])
    @convert_model
    def stat(self, params: WorkspaceGroupUserStatQueryRequest) -> dict:
        """Stat workspace group users
        Args:
            params (WorkspaceGroupUserStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_group_id': 'str',           # required
                'user_id': 'str'                       # injected from auth (required)
                'domain_id': 'str',                    # injected from auth (required)
            }
        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """
        query = params.query or {}

        return self.workspace_group_user_mgr.stat_workspace_group_users(
            query, params.workspace_group_id, params.domain_id
        )

    def _find(
        self, params: WorkspaceGroupUserFindRequest
    ) -> Union[WorkspaceGroupUsersSummaryResponse, dict]:
        workspace_group = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )
        workspace_group_users = workspace_group.users or []
        workspace_group_user_ids = list(
            set([user["user_id"] for user in workspace_group_users])
        )

        query = {
            "filter": [
                {"k": "domain_id", "v": params.domain_id, "o": "eq"},
                {"k": "user_id", "v": workspace_group_user_ids, "o": "not_in"},
            ],
            "sort": [{"key": "user_id"}],
            "page": params.page,
            "only": ["user_id", "name", "state"],
        }

        if params.keyword:
            query["filter_or"] = [
                {"k": "user_id", "v": params.keyword, "o": "contain"},
                {"k": "name", "v": params.keyword, "o": "contain"},
            ]

        if params.state:
            query["filter"].append({"k": "state", "v": params.state, "o": "eq"})

        user_mgr = UserManager()
        user_vos, total_count = user_mgr.list_users(query)

        workspace_group_users_info = [user_vo.to_dict() for user_vo in user_vos]

        return WorkspaceGroupUsersSummaryResponse(
            results=workspace_group_users_info, total_count=total_count
        )



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class CostAnalysisManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cost_analysis_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="cost_analysis"
        )

    def list_cost_reports(
        self, params: dict, token: str = None, x_domain_id: str = None
    ) -> dict:
        return self.cost_analysis_conn.dispatch(
            "CostReport.list",
            params,
            token=token,
            x_domain_id=x_domain_id,
        )



import logging
from typing import Tuple
from mongoengine import QuerySet

from spaceone.core.manager import BaseManager
from spaceone.identity.model.role_binding.database import RoleBinding

_LOGGER = logging.getLogger(__name__)


class RoleBindingManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_binding_model = RoleBinding

    def create_role_binding(self, params: dict) -> RoleBinding:
        def _rollback(vo: RoleBinding):
            _LOGGER.info(
                f"[create_role_binding._rollback] "
                f"Delete trusted service account: {vo.role_binding_id}"
            )
            vo.delete()

        role_binding_vo = self.role_binding_model.create(params)
        self.transaction.add_rollback(_rollback, role_binding_vo)

        return role_binding_vo

    def update_role_binding_by_vo(
        self, params: dict, role_binding_vo: RoleBinding
    ) -> RoleBinding:
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_role_binding_by_vo._rollback] Revert Data: "
                f'{old_data["role_binding_id"]}'
            )
            role_binding_vo.update(old_data)

        self.transaction.add_rollback(_rollback, role_binding_vo.to_dict())

        return role_binding_vo.update(params)

    @staticmethod
    def delete_role_binding_by_vo(role_binding_vo: RoleBinding) -> None:
        role_binding_vo.delete()

    def get_role_binding(
        self, role_binding_id: str, domain_id: str, workspace_id: str = None
    ) -> RoleBinding:

        conditions = {
            "role_binding_id": role_binding_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        return self.role_binding_model.get(**conditions)

    def filter_role_bindings(self, **conditions) -> QuerySet:
        return self.role_binding_model.filter(**conditions)

    def list_role_bindings(self, query: dict) -> Tuple[QuerySet, int]:
        return self.role_binding_model.query(**query)

    def stat_role_bindings(self, query: dict) -> dict:
        return self.role_binding_model.stat(**query)



import logging
import random
from abc import abstractmethod, ABC, ABCMeta
from collections import OrderedDict

from spaceone.core import config, utils, cache
from spaceone.core.manager import BaseManager

from spaceone.identity.error.error_authentication import ERROR_INVALID_CREDENTIALS
from spaceone.identity.error.error_mfa import ERROR_NOT_SUPPORTED_MFA_TYPE
from spaceone.identity.error.error_user import ERROR_INVALID_VERIFY_CODE

__all__ = ["BaseMFAManager", "MFAManager"]
_LOGGER = logging.getLogger(__name__)


class BaseMFAManager(BaseManager, ABC):
    @abstractmethod
    def enable_mfa(self, **kwargs):
        pass

    @abstractmethod
    def disable_mfa(self, **kwargs):
        pass

    def confirm_mfa(self, **kwargs):
        pass

    def _load_conf(self):
        identity_conf = config.get_global("IDENTITY") or {}
        mfa_conf = identity_conf.get("mfa", {})
        self.CONST_MFA_VERIFICATION_CODE_TIMEOUT = mfa_conf.get(
            "verify_code_timeout", 300
        )


class MFAManager(BaseMFAManager, metaclass=ABCMeta):
    mfa_type: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._load_conf()

    def enable_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.enable_mfa not implemented!")

    def disable_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.disable_mfa not implemented!")

    def confirm_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.confirm_mfa not implemented!")

    def create_mfa_verify_code(self, user_id: str, domain_id: str, credentials: dict):
        if cache.is_set():
            verify_code = self._generate_verify_code()
            ordered_credentials = OrderedDict(sorted(credentials.items()))
            hashed_credentials = utils.dict_to_hash(ordered_credentials)
            cache.delete(f"identity:mfa:{hashed_credentials}")
            cache.set(
                f"identity:mfa:{hashed_credentials}",
                {
                    "verify_code": verify_code,
                    "user_id": user_id,
                    "domain_id": domain_id,
                },
                expire=self.CONST_MFA_VERIFICATION_CODE_TIMEOUT,
            )
            return verify_code

    @classmethod
    def get_manager_by_mfa_type(cls, mfa_type):
        for subclass in cls.__subclasses__():
            if subclass.mfa_type == mfa_type:
                return subclass()
        raise ERROR_NOT_SUPPORTED_MFA_TYPE(support_mfa_types=["EMAIL"])

    @staticmethod
    def check_mfa_verify_code(credentials: dict, verify_code: str) -> bool:
        if cache.is_set():
            ordered_credentials = OrderedDict(sorted(credentials.items()))
            hashed_credentials = utils.dict_to_hash(ordered_credentials)
            cached_mfa_info = cache.get(f"identity:mfa:{hashed_credentials}")
            if cached_mfa_info["verify_code"] == verify_code:
                cache.delete(f"identity:mfa:{hashed_credentials}")
                return True
        raise ERROR_INVALID_VERIFY_CODE(verify_code=credentials["verify_code"])

    @staticmethod
    def get_mfa_info(credentials: dict):
        if cache.is_set():
            ordered_credentials = OrderedDict(sorted(credentials.items()))
            hashed_credentials = utils.dict_to_hash(ordered_credentials)
            cached_mfa_info = cache.get(f"identity:mfa:{hashed_credentials}")
            return cached_mfa_info
        raise ERROR_INVALID_CREDENTIALS()

    @staticmethod
    def _generate_verify_code():
        return str(random.randint(100000, 999999))



import logging
from typing import Tuple

from spaceone.core.manager import BaseManager

from spaceone.identity.connector.account_collector_plugin_connector import (
    AccountCollectorPluginConnector,
)
from spaceone.identity.manager.plugin_manager import PluginManager
from spaceone.identity.manager.provider_manager import ProviderManager
from spaceone.identity.model.provider.database import Provider
from spaceone.core.connector.space_connector import SpaceConnector

__ALL__ = ["AccountCollectorPluginManager"]

_LOGGER = logging.getLogger(__name__)


class AccountCollectorPluginManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.acp_connector = AccountCollectorPluginConnector()

    def initialize(self, endpoint: str) -> None:
        _LOGGER.debug(f"[initialize] account collector plugin endpoint: {endpoint}")
        self.acp_connector.initialize(endpoint)

    def init_plugin(self, domain_id: str, options: dict) -> dict:
        plugin_info = self.acp_connector.init(domain_id, options)

        _LOGGER.debug(f"[plugin_info] {plugin_info}")
        plugin_metadata = plugin_info.get("metadata", {})

        return plugin_metadata

    def sync(
        self,
        endpoint: str,
        options: dict,
        secret_data: dict,
        domain_id: str,
        schema_id: str = None,
    ) -> dict:
        plugin_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", endpoint=endpoint, token="NO_TOKEN"
        )

        params = {
            "options": options,
            "secret_data": secret_data,
            "domain_id": domain_id,
        }
        if schema_id:
            params["schema_id"] = schema_id

        return plugin_connector.dispatch("AccountCollector.sync", params)

    def get_account_collector_plugin_endpoint_by_vo(self, provider_vo: Provider) -> str:
        plugin_info = provider_vo.plugin_info
        endpoint, updated_version = self.get_account_collector_plugin_endpoint(
            plugin_info, provider_vo.domain_id
        )

        if updated_version:
            _LOGGER.debug(
                f'[get_account_collector_plugin_endpoint_by_vo] upgrade plugin version: {plugin_info.get("version")} -> {updated_version}'
            )
            self.upgrade_account_collector_plugin_version(
                provider_vo, endpoint, updated_version
            )

        return endpoint

    def get_account_collector_plugin_endpoint(
        self, plugin_info: dict, domain_id: str
    ) -> Tuple[str, str]:
        plugin_mgr: PluginManager = self.locator.get_manager("PluginManager")
        return plugin_mgr.get_plugin_endpoint(plugin_info, domain_id)

    def upgrade_account_collector_plugin_version(
        self, provider_vo: Provider, endpoint: str, updated_version: str
    ) -> None:
        plugin_info = provider_vo.plugin_info
        domain_id = provider_vo.domain_id

        self.initialize(endpoint)

        plugin_options = plugin_info.get("options", {})

        plugin_metadata = self.init_plugin(domain_id, plugin_options)
        plugin_info["version"] = updated_version
        plugin_info["metadata"] = plugin_metadata

        provider_mgr = ProviderManager()
        provider_mgr.update_provider_by_vo({"plugin_info": plugin_info}, provider_vo)



from datetime import datetime, timedelta
from typing import Union, Tuple

from spaceone.core.error import ERROR_EXIST_RESOURCE
from spaceone.core.service import (
    authentication_handler,
    authorization_handler,
    mutation_handler,
)
from spaceone.core.service import (
    BaseService,
    event_handler,
    transaction,
    convert_model,
    append_query_filter,
    append_keyword_filter,
)

from spaceone.identity.manager.agent_manager import AgentManager
from spaceone.identity.manager.app_manager import AppManager
from spaceone.identity.manager.client_secret_manager import ClientSecretManager
from spaceone.identity.manager.service_account_manager import ServiceAccountManager
from spaceone.identity.model import App, Agent
from spaceone.identity.model.agent.request import (
    AgentCreateRequest,
    AgentEnableRequest,
    AgentDisableRequest,
    AgentRegenerateRequest,
    AgentDeleteRequest,
    AgentGetRequest,
    AgentSearchQueryRequest,
)
from spaceone.identity.model.agent.response import AgentResponse, AgentsResponse


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class AgentService(BaseService):
    resource = "Agent"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.agent_mgr = AgentManager()
        self.service_account_mgr = ServiceAccountManager()
        self.app_mgr = AppManager()

    @transaction(
        permission="identity:Agent.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def create(self, params: AgentCreateRequest) -> Union[AgentResponse, dict]:
        """create agent using service account

         Args:
            params (AgentCreateRequest): {
                'service_account_id': 'str',            # required
                'options': 'dict',
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            AgentResponse:
        """
        service_account_id = params.service_account_id
        options = params.options or {}
        workspace_id = params.workspace_id
        domain_id = params.domain_id
        user_projects = params.user_projects

        service_account_vo = self.service_account_mgr.get_service_account(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        agent_vos = self.agent_mgr.filter_agents(
            domain_id=domain_id,
            workspace_id=workspace_id,
            project_id=service_account_vo.project_id,
            service_account_id=service_account_id,
        )
        if agent_vos:
            raise ERROR_EXIST_RESOURCE(
                child="Agent", parent=f"service_account({service_account_id})"
            )

        create_app_params = {
            "name": f"{service_account_vo.name} App",
            "is_managed": True,
            "role_type": "WORKSPACE_OWNER",
            "role_id": "managed-workspace-owner",
            "resource_group": "WORKSPACE",
            "domain_id": domain_id,
            "workspace_id": workspace_id,
            "service_account_id": service_account_id,
            "options": options,
            "expired_at": self._get_expired_at(),
        }

        app_vo = self.app_mgr.create_app(create_app_params)

        client_id, client_secret = self._create_agent_client_secret(
            app_vo, service_account_id
        )

        app_vo = self.app_mgr.update_app_by_vo({"client_id": client_id}, app_vo)

        create_agent_params = {
            "app_id": app_vo.app_id,
            "service_account_id": service_account_id,
            "project_id": service_account_vo.project_id,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
            "options": options,
            "expired_at": self._get_expired_at(),
        }

        agent_vo = self.agent_mgr.create_agent(create_agent_params)

        agent_info = self._update_agent_info_with_app_vo(agent_vo, app_vo)

        return AgentResponse(**agent_info, client_secret=client_secret)

    @transaction(
        permission="identity:Agent.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def enable(self, params: AgentEnableRequest) -> Union[AgentResponse, dict]:
        """enable agent created by service account

         Args:
            params (AgentEnableRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            AgentResponse:
        """
        service_account_id = params.service_account_id
        domain_id = params.domain_id
        workspace_id = params.workspace_id
        user_projects = params.user_projects

        self.service_account_mgr.get_service_account(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        agent_vo = self.agent_mgr.get_agent(
            service_account_id, domain_id, workspace_id, user_projects
        )
        app_vo = self.app_mgr.get_app(agent_vo.app_id, domain_id, workspace_id)
        app_vo = self.app_mgr.enable_app(app_vo)

        agent_info = self._update_agent_info_with_app_vo(agent_vo, app_vo)

        return AgentResponse(**agent_info)

    @transaction(
        permission="identity:Agent.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def disable(self, params: AgentDisableRequest) -> Union[AgentResponse, dict]:
        """disable agent created by service account

         Args:
            params (AgentDisableRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            AgentResponse:
        """
        service_account_id = params.service_account_id
        domain_id = params.domain_id
        workspace_id = params.workspace_id
        user_projects = params.user_projects

        self.service_account_mgr.get_service_account(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        agent_vo = self.agent_mgr.get_agent(
            service_account_id, domain_id, workspace_id, user_projects
        )
        app_vo = self.app_mgr.get_app(agent_vo.app_id, domain_id, workspace_id)
        app_vo = self.app_mgr.disable_app(app_vo)

        agent_info = self._update_agent_info_with_app_vo(agent_vo, app_vo)

        return AgentResponse(**agent_info)

    @transaction(
        permission="identity:Agent.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def regenerate(self, params: AgentRegenerateRequest) -> Union[AgentResponse, dict]:
        """regenerate agent created by service account

         Args:
            params (ServiceAccountRegenerateAppRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            AgentResponse:
        """
        service_account_id = params.service_account_id
        domain_id = params.domain_id
        workspace_id = params.workspace_id
        user_projects = params.user_projects

        self.service_account_mgr.get_service_account(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        agent_vo = self.agent_mgr.get_agent(
            service_account_id, domain_id, workspace_id, user_projects
        )

        app_vo = self.app_mgr.get_app(agent_vo.app_id, domain_id, workspace_id)

        client_id, client_secret = self._create_agent_client_secret(
            app_vo, service_account_id
        )

        app_vo = self.app_mgr.update_app_by_vo({"client_id": client_id}, app_vo)

        agent_info = self._update_agent_info_with_app_vo(agent_vo, app_vo)

        return AgentResponse(**agent_info, client_secret=client_secret)

    @transaction(
        permission="identity:Agent.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def delete(self, params: AgentDeleteRequest) -> None:
        """delete agent created by service account

        Args:
            params (AgentDeleteRequest): {
                'service_account_id: 'str'   # required
                'workspace_id': 'str',       # injected from auth (required)
                'domain_id': 'str'           # injected from auth (required)
                'user_projects': 'list'      # injected from auth
            }
        Returns:
            None
        """
        service_account_id = params.service_account_id
        domain_id = params.domain_id
        workspace_id = params.workspace_id
        user_projects = params.user_projects

        self.service_account_mgr.get_service_account(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        agent_vo: Agent = self.agent_mgr.get_agent(
            service_account_id, domain_id, workspace_id, user_projects
        )

        app_vo = self.app_mgr.get_app(
            agent_vo.app_id,
            domain_id,
            workspace_id,
        )
        self.app_mgr.delete_app_by_vo(app_vo)
        self.agent_mgr.delete_agent_by_vo(agent_vo)

    @transaction(
        permission="identity:Agent.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(self, params: AgentGetRequest) -> Union[AgentResponse, dict]:
        """Get agent info
        Args:
            params (dict): {
                'service_account_id': 'str',          # required
                'workspace_id': 'list',               # injected from auth
                'domain_id': 'str'                    # injected from auth (required)
                'user_projects': 'list'               # injected from auth
            }
        Returns:
            AgentResponse:
        """
        service_account_id = params.service_account_id
        domain_id = params.domain_id
        workspace_id = params.workspace_id
        user_projects = params.user_projects

        agent_vo = self.agent_mgr.get_agent(
            service_account_id,
            domain_id,
            workspace_id,
            user_projects,
        )

        app_vo = self.app_mgr.get_app(agent_vo.app_id, domain_id, workspace_id)

        agent_info = self._update_agent_info_with_app_vo(agent_vo, app_vo)

        return AgentResponse(**agent_info)

    @transaction(
        permission="identity:Agent.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(
        [
            "agent_id",
            "service_account_id",
            "state",
            "workspace_id",
            "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["agent_id", "name"])
    @convert_model
    def list(self, params: AgentSearchQueryRequest) -> Union[AgentsResponse, dict]:
        """List Agents
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'agent_id': 'str',
                'state': 'str',
                'service_account_id': 'str',  # required
                'workspace_id': 'list'        # injected from auth
                'domain_id': 'str'            # injected from auth (required)
                'user_projects': 'list'       # injected from auth
            }
        Returns:
            AgentsResponse:
        """
        query = params.query or {}
        agent_vos, agent_total_count = self.agent_mgr.list_agents(query)
        agents_info = [agent_vo.to_dict() for agent_vo in agent_vos]

        app_ids = [agent_vo.app_id for agent_vo in agent_vos]

        app_vos, app_total_count = self.app_mgr.list_apps(
            {"filter": [{"k": "app_id", "v": app_ids, "o": "in"}]}
        )

        apps_info = [app_vo.to_dict() for app_vo in app_vos]

        agents_info = self._update_agents_info_with_apps_info(agents_info, apps_info)

        return AgentsResponse(results=agents_info, total_count=len(agents_info))

    def _create_agent_client_secret(
        self, app_vo: App, service_account_id: str
    ) -> Tuple[str, str]:
        """create client_id, client_secret for agent created by service account

        Args:
            app_vo: 'App'                   # required
            service_account_id: 'str'       # required

        Returns:
            tuple: ('client_id': 'str', 'client_secret': 'str')
        """
        permissions = ["identity:ServiceAccount.read"]
        injected_params = {"service_account_id": service_account_id}
        expired_at = self._get_expired_at()

        client_secret_mgr = ClientSecretManager()
        client_id, client_secret = client_secret_mgr.generate_client_secret(
            app_vo.app_id,
            app_vo.domain_id,
            expired_at,
            app_vo.role_type,
            app_vo.workspace_id,
            permissions=permissions,
            injected_params=injected_params,
        )

        return client_id, client_secret

    @staticmethod
    def _update_agent_info_with_app_vo(agent_vo, app_vo):
        agent_info = agent_vo.to_dict()
        agent_info["state"] = app_vo.state
        agent_info["is_managed"] = app_vo.is_managed
        agent_info["role_type"] = app_vo.role_type
        agent_info["role_id"] = app_vo.role_id
        agent_info["app_id"] = app_vo.app_id
        agent_info["client_id"] = app_vo.client_id
        agent_info["expired_at"] = app_vo.expired_at
        agent_info["last_accessed_at"] = app_vo.last_accessed_at

        return agent_info

    @staticmethod
    def _update_agents_info_with_apps_info(agents_info, apps_info):
        agents_info_with_apps_info = []

        for agent_info, app_info in zip(agents_info, apps_info):
            agent_info.update(
                {
                    "state": app_info["state"],
                    "is_managed": app_info["is_managed"],
                    "role_type": app_info["role_type"],
                    "role_id": app_info["role_id"],
                    "app_id": app_info["app_id"],
                    "client_id": app_info["client_id"],
                    "expired_at": app_info["expired_at"],
                    "last_accessed_at": app_info["last_accessed_at"],
                }
            )
            agents_info_with_apps_info.append(agent_info)

        return agents_info_with_apps_info

    @staticmethod
    def _get_expired_at() -> str:
        return (datetime.utcnow() + timedelta(days=365)).strftime("%Y-%m-%d %H:%M:%S")



from datetime import datetime
from typing import Union, List, Literal
from pydantic import BaseModel

from spaceone.core import utils

from spaceone.identity.model.user.request import State, AuthType

__all__ = [
    "UserResponse",
    "UsersResponse",
]

RoleType = Literal["DOMAIN_ADMIN", "USER"]


class UserResponse(BaseModel):
    user_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    email: Union[str, None] = None
    email_verified: Union[bool, None] = None
    auth_type: Union[AuthType, None] = None
    role_id: Union[str, None] = None
    role_type: Union[RoleType, None] = None
    mfa: Union[dict, None] = None
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    required_actions: Union[List[str], None] = None
    tags: Union[dict, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    last_accessed_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["last_accessed_at"] = utils.datetime_to_iso8601(data["last_accessed_at"])
        return data


class UsersResponse(BaseModel):
    results: List[UserResponse]
    total_count: int



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "AppCreateRequest",
    "AppUpdateRequest",
    "AppGenerateAPIKeyRequest",
    "AppEnableRequest",
    "AppDisableRequest",
    "AppDeleteRequest",
    "AppGetRequest",
    "AppCheckRequest",
    "AppSearchQueryRequest",
    "AppStatQueryRequest",
    "State",
    "ResourceGroup",
    "RoleType",
]

State = Literal["ENABLED", "DISABLED", "EXPIRED"]
ResourceGroup = Literal["DOMAIN", "WORKSPACE", "PROJECT"]
RoleType = Literal["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"]


class AppCreateRequest(BaseModel):
    name: str
    role_id: str
    tags: Union[dict, None] = None
    expired_at: Union[str, None] = None
    resource_group: ResourceGroup
    project_id: Union[str, None] = None
    project_group_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class AppUpdateRequest(BaseModel):
    app_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class AppGenerateAPIKeyRequest(BaseModel):
    app_id: str
    workspace_id: Union[str, None] = None
    expired_at: Union[str, None] = None
    domain_id: str


class AppEnableRequest(BaseModel):
    app_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class AppDisableRequest(BaseModel):
    app_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class AppDeleteRequest(BaseModel):
    app_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class AppGetRequest(BaseModel):
    app_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class AppCheckRequest(BaseModel):
    client_id: str
    domain_id: str


class AppSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    app_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    role_type: Union[str, None] = None
    role_id: Union[str, None] = None
    client_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class AppStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str



from spaceone.api.identity.v2 import workspace_group_pb2, workspace_group_pb2_grpc
from spaceone.core.pygrpc import BaseAPI

from spaceone.identity.service.workspace_group_service import WorkspaceGroupService


class WorkspaceGroup(BaseAPI, workspace_group_pb2_grpc.WorkspaceGroupServicer):
    pb2 = workspace_group_pb2
    pb2_grpc = workspace_group_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.update(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        workspace_group_svc.delete(params)
        return self.empty()

    def add_workspaces(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.add_workspaces(params)
        return self.dict_to_message(response)

    def remove_workspaces(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.remove_workspaces(params)
        return self.dict_to_message(response)

    def find_users(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.find_users(params)
        return self.dict_to_message(response)

    def add_users(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.add_users(params)
        return self.dict_to_message(response)

    def remove_users(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.remove_users(params)
        return self.dict_to_message(response)

    def update_role(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.update_role(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_svc = WorkspaceGroupService(metadata)
        response: dict = workspace_group_svc.stat(params)
        return self.dict_to_message(response)



from mongoengine import *
from datetime import datetime
from spaceone.core.error import *
from spaceone.core.model.mongo_model import MongoModel


class Domain(MongoModel):
    domain_id = StringField(max_length=40, generate_id="domain", unique=True)
    name = StringField(max_length=255)
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED", "DELETED")
    )
    tags = DictField(default=None)
    created_at = DateTimeField(auto_now_add=True)
    deleted_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "tags",
            "deleted_at",
        ],
        "minimal_fields": [
            "domain_id",
            "name",
            "state",
        ],
        "ordering": ["name"],
        "indexes": [
            # 'domain_id',
            "state",
        ],
    }

    @queryset_manager
    def objects(doc_cls, query_set: QuerySet):
        return query_set.filter(state__ne="DELETED")

    @classmethod
    def create(cls, data):
        domain_vos = cls.filter(name=data["name"])
        if domain_vos.count() > 0:
            raise ERROR_NOT_UNIQUE(key="name", value=data["name"])

        return super().create(data)

    def update(self, data):
        if "name" in data:
            domain_vos = self.filter(name=data["name"], domain_id__ne=self.domain_id)
            if domain_vos.count() > 0:
                raise ERROR_NOT_UNIQUE(key="name", value=data["name"])

        return super().update(data)

    def delete(self):
        self.update({"state": "DELETED", "deleted_at": datetime.utcnow()})


class DomainSecret(MongoModel):
    domain_key = StringField()
    pub_jwk = DictField(required=True)
    prv_jwk = DictField(required=True)
    refresh_pub_jwk = DictField(required=True)
    refresh_prv_jwk = DictField(required=True)
    domain_id = StringField(max_length=40, unique=True)
    domain = ReferenceField("Domain", reverse_delete_rule=CASCADE)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "ordering": ["domain_id"],
        "indexes": ["domain"],
    }



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.manager.project_group_manager import ProjectGroupManager
from spaceone.identity.manager.resource_manager import ResourceManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.manager.workspace_user_manager import WorkspaceUserManager
from spaceone.identity.model.project.request import *
from spaceone.identity.model.project.response import *
from spaceone.identity.error.error_project import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class ProjectService(BaseService):
    resource = "Project"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.rb_mgr = RoleBindingManager()
        self.project_mgr = ProjectManager()
        self.project_group_mgr = ProjectGroupManager()
        self.resource_mgr = ResourceManager()
        self.workspace_mgr = WorkspaceManager()

    @transaction(permission="identity:Project.write", role_types=["WORKSPACE_OWNER"])
    @convert_model
    def create(self, params: ProjectCreateRequest) -> Union[ProjectResponse, dict]:
        """Create project
        Args:
            params (ProjectCreateRequest): {
                'name': 'str',                  # required
                'project_type': 'str',          # required
                'tags': 'dict',
                'project_group_id': 'str',
                'workspace_id': 'str',          # injected from auth (required)
                'domain_id': 'str',             # injected from auth (required)
            }
        Returns:
        """

        if params.project_group_id:
            self.project_group_mgr.get_project_group(
                params.project_group_id,
                params.domain_id,
                params.workspace_id,
            )

        params_data = params.dict()
        params_data["created_by"] = self.transaction.get_meta("authorization.user_id")

        project_vo = self.project_mgr.create_project(params_data)

        return ProjectResponse(**project_vo.to_dict())

    @transaction(
        permission="identity:Project.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def update(self, params: ProjectUpdateRequest) -> Union[ProjectResponse, dict]:
        """Update project
        Args:
            params (ProjectUpdateRequest): {
                'project_id': 'str',        # required
                'name': 'str',
                'tags': 'dict',
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }
        Returns:
            ProjectResponse:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        # Check is managed resource
        self.resource_mgr.check_is_managed_resource_by_trusted_account(project_vo)

        project_vo = self.project_mgr.update_project_by_vo(
            params.dict(exclude_unset=True), project_vo
        )

        return ProjectResponse(**project_vo.to_dict())

    @transaction(permission="identity:Project.write", role_types=["WORKSPACE_OWNER"])
    @convert_model
    def update_project_type(
        self, params: ProjectUpdateProjectTypeRequest
    ) -> Union[ProjectResponse, dict]:
        """Update project type
        Args:
            params (ProjectUpdateProjectTypeRequest): {
                'project_id': 'str',        # required
                'project_type': 'str',      # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            ProjectResponse:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id, params.domain_id, params.workspace_id
        )

        # Check is managed resource
        self.resource_mgr.check_is_managed_resource_by_trusted_account(project_vo)

        params_dict = params.dict(exclude_unset=True)
        if params.project_type == "PUBLIC":
            params_dict["users"] = []

        project_vo = self.project_mgr.update_project_by_vo(params_dict, project_vo)

        return ProjectResponse(**project_vo.to_dict())

    @transaction(permission="identity:Project.write", role_types=["WORKSPACE_OWNER"])
    @convert_model
    def change_project_group(
        self, params: ProjectChangeProjectGroupRequest
    ) -> Union[ProjectResponse, dict]:
        """Change project group
        Args:
            params (ProjectChangeProjectGroupRequest): {
                'project_id': 'str',            # required
                'project_group_id': 'str',      # required
                'workspace_id': 'str',          # injected from auth (required)
                'domain_id': 'str'              # injected from auth (required)
            }
        Returns:
            ProjectResponse:
        """

        if params.project_group_id:
            self.project_group_mgr.get_project_group(
                params.project_group_id,
                params.domain_id,
                params.workspace_id,
            )

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
        )

        # Check is managed resource
        self.resource_mgr.check_is_managed_resource_by_trusted_account(project_vo)

        project_vo = self.project_mgr.update_project_by_vo(params.dict(), project_vo)

        return ProjectResponse(**project_vo.to_dict())

    @transaction(permission="identity:Project.write", role_types=["WORKSPACE_OWNER"])
    @convert_model
    def delete(self, params: ProjectDeleteRequest) -> None:
        """Delete project
        Args:
            params (ProjectDeleteRequest): {
                'project_id': 'str',        # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            None:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
        )

        # Check is managed resource
        self.resource_mgr.check_is_managed_resource_by_trusted_account(project_vo)

        self.project_mgr.delete_project_by_vo(project_vo)

    @transaction(
        permission="identity:Project.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def add_users(self, params: ProjectAddUsersRequest) -> Union[ProjectResponse, dict]:
        """Add users to project
        Args:
            params (ProjectAddUsersRequest): {
                'project_id': 'str',        # required
                'users': 'list',            # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }
        Returns:
            ProjectResponse:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        if project_vo.project_type == "PUBLIC":
            raise ERROR_NOT_ALLOWED_ADD_USER_TO_PUBLIC_PROJECT()

        if len(params.users) > 0:
            workspace_user_mgr = WorkspaceUserManager()
            workspace_user_mgr.check_workspace_users(
                params.users, params.workspace_id, params.domain_id
            )

            users = project_vo.users or []
            users.extend(params.users)
            params.users = list(set(users))

            project_vo = self.project_mgr.update_project_by_vo(
                params.dict(exclude_unset=True), project_vo
            )

        return ProjectResponse(**project_vo.to_dict())

    @transaction(
        permission="identity:Project.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def remove_users(
        self, params: ProjectRemoveUsersRequest
    ) -> Union[ProjectResponse, dict]:
        """Remove users from project
        Args:
            params (ProjectRemoveUsersRequest): {
                'project_id': 'str',        # required
                'users': 'list',            # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }
        Returns:
            ProjectResponse:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        if len(params.users) > 0:
            workspace_user_mgr = WorkspaceUserManager()
            workspace_user_mgr.check_workspace_users(
                params.users, params.workspace_id, params.domain_id
            )

            users = project_vo.users or []
            params.users = list(set(users) - set(params.users))

        project_vo = self.project_mgr.update_project_by_vo(
            params.dict(exclude_unset=True), project_vo
        )

        return ProjectResponse(**project_vo.to_dict())

    @transaction(
        permission="identity:Project.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(self, params: ProjectGetRequest) -> Union[ProjectResponse, dict]:
        """Get project
        Args:
            params (ProjectGetRequest): {
                'project_id': 'str',    # required
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str',     # injected from auth (required)
                'user_projects': 'list' # injected from auth
            }
        Returns:
            ProjectResponse:
        """

        project_vo = self.project_mgr.get_project(
            params.project_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        return ProjectResponse(**project_vo.to_dict())

    @transaction(
        permission="identity:Project.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(
        [
            "project_id",
            "name",
            "project_type",
            "created_by",
            "user_id",
            "workspace_id",
            "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["project_id", "name"])
    @convert_model
    def list(self, params: ProjectSearchQueryRequest) -> Union[ProjectsResponse, dict]:
        """List projects
        Args:
            params (ProjectSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'project_id': 'str',
                'name': 'str',
                'project_type': 'str',
                'created_by': 'str',
                'include_children': 'bool',
                'user_id': 'str',
                'project_group_id': 'str',
                'workspace_id': 'str',      # injected from auth
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }
        Returns:
            ProjectsResponse:
        """

        include_children = params.include_children or False
        project_group_id = params.project_group_id
        query = params.query or {}

        if include_children and project_group_id:
            project_group_ids = self._get_child_group_ids(project_group_id)
            project_group_ids.append(project_group_id)
            query["filter"].append(
                {"k": "project_group_id", "v": project_group_ids, "o": "in"}
            )
        elif project_group_id:
            query["filter"].append(
                {"k": "project_group_id", "v": project_group_id, "o": "eq"}
            )

        project_vos, total_count = self.project_mgr.list_projects(query)

        projects_info = [project_vo.to_dict() for project_vo in project_vos]
        return ProjectsResponse(results=projects_info, total_count=total_count)

    @transaction(
        permission="identity:Project.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["workspace_id", "domain_id", "user_projects"])
    @append_keyword_filter(["project_id", "name"])
    @convert_model
    def stat(self, params: ProjectStatQueryRequest) -> dict:
        """Stat workspaces
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',      # injected from auth
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }
        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """

        query = params.query or {}
        return self.project_mgr.stat_projects(query)

    def _get_child_group_ids(self, project_group_id: str) -> list:
        pg_vos = self.project_group_mgr.filter_project_groups(
            parent_group_id=project_group_id
        )
        project_group_ids = [pg.project_group_id for pg in pg_vos]

        for pg in pg_vos:
            project_group_ids.extend(self._get_child_group_ids(pg.project_group_id))

        return project_group_ids



from datetime import datetime
from typing import Dict, List, Union

from pydantic.main import BaseModel
from spaceone.core import utils

__all__ = ["WorkspaceGroupResponse", "WorkspaceGroupsResponse"]


class WorkspaceGroupResponse(BaseModel):
    workspace_group_id: Union[str, None] = None
    name: Union[str, None] = None
    workspaces: Union[list, None] = None
    users: Union[List[Dict[str, str]], None] = None
    tags: Union[dict, None] = None
    created_by: Union[str, None] = None
    updated_by: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["updated_at"] = utils.datetime_to_iso8601(data["updated_at"])
        return data


class WorkspaceGroupsResponse(BaseModel):
    results: List[WorkspaceGroupResponse]
    total_count: int



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel

from spaceone.core import utils

from spaceone.identity.model.app.request import State, ResourceGroup, RoleType

__all__ = ["CheckAppResponse", "AppResponse", "AppsResponse"]


class CheckAppResponse(BaseModel):
    permissions: List[str]
    projects: Union[List[str], None] = None


class AppResponse(BaseModel):
    app_id: Union[str, None] = None
    client_secret: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    tags: Union[dict, None] = None
    is_managed: Union[bool, None] = None
    role_type: Union[RoleType, None] = None
    client_id: Union[str, None] = None
    role_id: Union[str, None] = None
    resource_group: Union[ResourceGroup, None] = None
    project_id: Union[str, None] = None
    project_group_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    expired_at: Union[datetime, None] = None
    last_accessed_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["expired_at"] = utils.datetime_to_iso8601(data["expired_at"])
        data["last_accessed_at"] = utils.datetime_to_iso8601(
            data.get("last_accessed_at")
        )
        return data


class AppsResponse(BaseModel):
    results: List[AppResponse]
    total_count: int



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ServiceAccount(MongoModel):
    service_account_id = StringField(max_length=40, generate_id="sa", unique=True)
    name = StringField(
        max_length=255, unique_with=["domain_id", "workspace_id", "project_id"]
    )
    state = StringField(
        max_length=20,
        required=True,
        choices=["PENDING", "ACTIVE", "INACTIVE", "DELETED"],
    )
    data = DictField(default=None)
    provider = StringField(max_length=40)
    tags = DictField(default=None)
    reference_id = StringField(max_length=255, default=None, null=True)
    is_managed = BooleanField(default=False)
    cost_info = DictField(default=None)
    secret_schema_id = StringField(max_length=40)
    secret_id = StringField(max_length=40)
    trusted_account_id = StringField(max_length=40, null=True, default=None)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    last_synced_at = DateTimeField(default=None, null=True)
    deleted_at = DateTimeField(default=None, null=True)
    inactivated_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "data",
            "tags",
            "is_managed",
            "cost_info",
            "secret_schema_id",
            "secret_id",
            "trusted_account_id",
            "project_id",
            "last_synced_at",
            "inactivated_at",
        ],
        "minimal_fields": [
            "service_account_id",
            "name",
            "state",
            "provider",
            "is_managed",
            "trusted_account_id",
            "project_id",
            "workspace_id",
        ],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["name"],
        "indexes": [
            "name",
            "state",
            "provider",
            "secret_schema_id",
            "secret_id",
            "trusted_account_id",
            "project_id",
            "workspace_id",
            "domain_id",
        ],
    }



from spaceone.api.identity.v2 import (
    workspace_group_user_pb2,
    workspace_group_user_pb2_grpc,
)
from spaceone.core.pygrpc import BaseAPI

from spaceone.identity.service.workspace_group_user_service import (
    WorkspaceGroupUserService,
)


class WorkspaceGroupUser(
    BaseAPI, workspace_group_user_pb2_grpc.WorkspaceGroupUserServicer
):
    pb2 = workspace_group_user_pb2
    pb2_grpc = workspace_group_user_pb2_grpc

    def add(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.add(params)
        return self.dict_to_message(response)

    def remove(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.remove(params)
        return self.dict_to_message(response)

    def update_role(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.update_role(params)
        return self.dict_to_message(response)

    def find(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.find(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_group_user_svc = WorkspaceGroupUserService(metadata)
        response: dict = workspace_group_user_svc.stat(params)
        return self.dict_to_message(response)



import logging
from typing import Tuple, List
from mongoengine import QuerySet

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.identity.model.service_account.database import ServiceAccount

_LOGGER = logging.getLogger(__name__)


class ServiceAccountManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.service_account_model = ServiceAccount

    def create_service_account(self, params: dict) -> ServiceAccount:
        def _rollback(vo: ServiceAccount):
            _LOGGER.info(
                f"[create_service_account._rollback] "
                f"Delete service account: {vo.name} ({vo.service_account_id})"
            )
            service_account_vo.delete()

        params["state"] = "ACTIVE"
        params["cost_info"] = {
            "day": 0,
            "month": 0,
        }

        service_account_vo = self.service_account_model.create(params)
        self.transaction.add_rollback(_rollback, service_account_vo)

        return service_account_vo

    def update_service_account(self, params: dict) -> ServiceAccount:
        service_account_vo = self.get_service_account(
            params["service_account_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

        return self.update_service_account_by_vo(params, service_account_vo)

    def update_service_account_by_vo(
        self, params: dict, service_account_vo: ServiceAccount
    ) -> ServiceAccount:
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_service_account_by_vo._rollback] Revert Data : "
                f'{old_data["service_account_id"]}'
            )
            service_account_vo.update(old_data)

        self.transaction.add_rollback(_rollback, service_account_vo.to_dict())

        return service_account_vo.update(params)

    @staticmethod
    def delete_service_account_by_vo(service_account_vo: ServiceAccount) -> None:
        service_account_vo.delete()

    def get_service_account(
        self,
        service_account_id: str,
        domain_id: str,
        workspace_id: str = None,
        user_projects: List[str] = None,
    ) -> ServiceAccount:
        conditions = {"service_account_id": service_account_id, "domain_id": domain_id}

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if user_projects:
            conditions["project_id"] = user_projects

        return self.service_account_model.get(**conditions)

    def filter_service_accounts(self, **conditions) -> QuerySet:
        return self.service_account_model.filter(**conditions)

    def list_service_accounts(self, query: dict) -> Tuple[list, int]:
        return self.service_account_model.query(**query)

    def stat_service_accounts(self, query: dict) -> dict:
        return self.service_account_model.stat(**query)

    def update_secret_project(
        self,
        service_account_id: str,
        domain_id: str,
        workspace_id: str,
        project_id: str,
    ) -> None:
        secret_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="secret"
        )
        response = self._list_secrets(
            secret_connector, service_account_id, domain_id, workspace_id
        )

        for secret_info in response.get("results", []):
            secret_connector.dispatch(
                "Secret.update",
                {
                    "secret_id": secret_info["secret_id"],
                    "project_id": project_id,
                    "domain_id": domain_id,
                    "workspace_id": workspace_id,
                },
            )

    def delete_secrets(
        self, service_account_id: str, domain_id: str, workspace_id: str
    ) -> None:
        secret_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="secret"
        )
        response = self._list_secrets(
            secret_connector, service_account_id, domain_id, workspace_id
        )

        for secret_info in response.get("results", []):
            secret_connector.dispatch(
                "Secret.delete",
                {
                    "secret_id": secret_info["secret_id"],
                    "domain_id": domain_id,
                    "workspace_id": workspace_id,
                },
            )

    def get_all_service_account_ids_using_secret(
        self, domain_id: str, workspace_id: str
    ) -> List[str]:
        secret_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="secret"
        )
        response = secret_connector.dispatch(
            "Secret.stat",
            {
                "query": {
                    "distinct": "service_account_id",
                    "filter": [{"k": "service_account_id", "v": None, "o": "not"}],
                },
                "domain_id": domain_id,
                "workspace_id": workspace_id,
            },
        )

        return response.get("results", [])

    @staticmethod
    def _list_secrets(
        secret_connector: SpaceConnector,
        service_account_id: str,
        domain_id: str,
        workspace_id: str,
    ) -> dict:
        return secret_connector.dispatch(
            "Secret.list",
            {
                "service_account_id": service_account_id,
                "domain_id": domain_id,
                "workspace_id": workspace_id,
            },
        )



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ProjectGroup(MongoModel):
    project_group_id = StringField(max_length=40, generate_id="pg", unique=True)
    name = StringField(max_length=255)
    tags = DictField(default=None)
    users = ListField(StringField(max_length=255), default=None)
    reference_id = StringField(max_length=255, default=None, null=True)
    is_managed = BooleanField(default=False)
    trusted_account_id = StringField(max_length=40, default=None, null=True)
    parent_group_id = StringField(max_length=40, null=True, default=None)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    last_synced_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "name",
            "tags",
            "users",
            "is_managed",
            "trusted_account_id",
            "parent_group_id",
            "last_synced_at",
        ],
        "minimal_fields": [
            "project_group_id",
            "name",
            "parent_group_id",
            "workspace_id",
        ],
        "change_query_keys": {
            "user_id": "users",
        },
        "ordering": ["name"],
        "indexes": [
            "parent_group_id",
            "workspace_id",
            "domain_id",
        ],
    }



import logging
import random
from abc import abstractmethod, ABC
from datetime import datetime
from typing import Union

from spaceone.core import config, cache
from spaceone.core.manager import BaseManager
from spaceone.identity.error.error_authentication import *

from spaceone.identity.error.error_token import *
from spaceone.identity.lib.key_generator import KeyGenerator

__all__ = ["TokenManager"]
_LOGGER = logging.getLogger(__name__)


class TokenManager(BaseManager, ABC):
    is_authenticated = False
    auth_type = None
    owner_type = "USER"
    user = None
    app = None
    role_type = "USER"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._load_conf()

    @abstractmethod
    def authenticate(self, domain_id, *args, **kwargs):
        pass

    @classmethod
    def get_token_manager_by_auth_type(cls, auth_type):
        for subclass in cls.__subclasses__():
            if subclass.auth_type == auth_type:
                return subclass()
        raise ERROR_INVALID_AUTHENTICATION_TYPE(auth_type=auth_type)

    def issue_token(
        self,
        private_jwk,
        refresh_private_jwk,
        domain_id,
        workspace_id=None,
        timeout=None,
        permissions=None,
        projects=None,
        app_id=None,
    ):
        if self.is_authenticated is False:
            raise ERROR_NOT_AUTHENTICATED()

        if self.owner_type == "USER":
            audience = self.user.user_id
        elif self.owner_type == "SYSTEM":
            # todo : remove
            audience = app_id
        else:
            audience = self.app.app_id

        key_gen = KeyGenerator(
            private_jwk,
            domain_id,
            self.owner_type,
            audience,
            refresh_prv_jwk=refresh_private_jwk,
        )

        timeout = self.set_timeout(timeout)

        access_token = key_gen.generate_token(
            "ACCESS_TOKEN",
            timeout=timeout,
            role_type=self.role_type,
            workspace_id=workspace_id,
            permissions=permissions,
            projects=projects,
        )

        refresh_token = key_gen.generate_token(
            "REFRESH_TOKEN", timeout=self.CONST_REFRESH_TIMEOUT
        )
        if self.owner_type != "SYSTEM":
            # todo: remove
            self.user.update({"last_accessed_at": datetime.utcnow()})

        return {"access_token": access_token, "refresh_token": refresh_token}

    def issue_temporary_token(
        self, user_id: str, domain_id: str, private_jwk, timeout: int
    ) -> dict:
        permissions = [
            "identity:UserProfile",
        ]

        key_gen = KeyGenerator(
            private_jwk,
            domain_id,
            self.owner_type,
            user_id,
        )

        # Issue token
        access_token = key_gen.generate_token(
            "ACCESS_TOKEN", timeout=timeout, role_type="USER", permissions=permissions
        )

        return {"access_token": access_token}

    def create_verify_code(self, user_id, domain_id):
        if cache.is_set():
            verify_code = self._generate_verify_code()
            cache.delete(f"identity:verify-code:{domain_id}:{user_id}")
            cache.set(
                f"identity:verify-code:{domain_id}:{user_id}",
                verify_code,
                expire=self.CONST_VERIFY_CODE_TIMEOUT,
            )
            return verify_code

    def set_timeout(self, timeout: Union[int, None]) -> int:
        if timeout and timeout > 0:
            timeout = min(timeout, self.CONST_MAX_TOKEN_TIMEOUT)
        else:
            timeout = self.CONST_TOKEN_TIMEOUT
        return timeout

    @staticmethod
    def check_verify_code(user_id, domain_id, verify_code):
        if cache.is_set():
            cached_verify_code = cache.get(
                f"identity:verify-code:{domain_id}:{user_id}"
            )
            if cached_verify_code == verify_code:
                return True
        return False

    @staticmethod
    def _generate_verify_code():
        return str(random.randint(100000, 999999))

    def _load_conf(self):
        identity_conf = config.get_global("IDENTITY") or {}
        token_conf = identity_conf.get("token", {})
        self.CONST_TOKEN_TIMEOUT = token_conf.get("token_timeout", 1800)
        self.CONST_VERIFY_CODE_TIMEOUT = token_conf.get("verify_code_timeout", 3600)
        self.CONST_REFRESH_TIMEOUT = token_conf.get("refresh_timeout", 10800)
        self.CONST_MAX_TOKEN_TIMEOUT = token_conf.get("token_max_timeout", 604800)



from typing import Union
from pydantic import BaseModel

__all__ = ["AccountCollectorInitRequest", "AccountCollectorSyncRequest"]


class AccountCollectorInitRequest(BaseModel):
    options: dict
    domain_id: str


class AccountCollectorSyncRequest(BaseModel):
    options: dict
    schema_id: Union[dict, None] = None
    secret_data: dict
    domain_id: str



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import endpoint_pb2, endpoint_pb2_grpc
from spaceone.identity.service.endpoint_service import EndpointService


class Endpoint(BaseAPI, endpoint_pb2_grpc.EndpointServicer):
    pb2 = endpoint_pb2
    pb2_grpc = endpoint_pb2_grpc

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        endpoint_svc = EndpointService(metadata)
        response: dict = endpoint_svc.list(params)
        return self.dict_to_message(response)



from typing import Union

from pydantic import BaseModel


class PluginResponse(BaseModel):
    metadata: dict


class UserResponse(BaseModel):
    state: Union[str, None] = None
    user_id: Union[str, None] = None
    name: Union[str, None] = None
    email: Union[str, None] = None
    mobile: Union[str, None] = None
    group: Union[str, None] = None



import logging
from typing import Union

from spaceone.core.error import *
from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.model.role.request import *
from spaceone.identity.model.role.request import BasicRoleSearchQueryRequest
from spaceone.identity.model.role.response import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class RoleService(BaseService):
    resource = "Role"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_mgr = RoleManager()

    @transaction(permission="identity:Role.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def create(self, params: RoleCreateRequest) -> Union[RoleResponse, dict]:
        """create role

         Args:
            params (RoleCreateRequest): {
                'name': 'str',                          # required
                'role_type': 'list',                    # required
                'permissions': 'list',
                'page_access': 'list',
                'tags': 'dict',
                'domain_id': 'str'                      # injected from auth (required)
            }

        Returns:
            RoleResponse:
        """

        role_vo = self.role_mgr.create_role(params.dict())
        return RoleResponse(**role_vo.to_dict())

    @transaction(permission="identity:Role.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def update(self, params: RoleUpdateRequest) -> Union[RoleResponse, dict]:
        """update role

         Args:
            params (RoleUpdateRequest): {
                'role_id': 'str',           # required
                'name': 'str',
                'permissions': 'list',
                'page_access': 'list',
                'tags': 'dict',
                'domain_id': 'str'          # injected from auth (required)
            }

        Returns:
            RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        if role_vo.is_managed:
            raise ERROR_PERMISSION_DENIED()

        role_vo = self.role_mgr.update_role_by_vo(
            params.dict(exclude_unset=True), role_vo
        )

        return RoleResponse(**role_vo.to_dict())

    @transaction(permission="identity:Role.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def enable(self, params: RoleEnableRequest) -> Union[RoleResponse, dict]:
        """enable role

         Args:
            params (RoleEnableRequest): {
                'role_id': 'str',           # required
                'domain_id': 'str',         # injected from auth (required)
            }
        Returns:
            RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        role_vo = self.role_mgr.enable_role_by_vo(role_vo)

        return RoleResponse(**role_vo.to_dict())

    @transaction(permission="identity:Role.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def disable(self, params: RoleDisableRequest) -> Union[RoleResponse, dict]:
        """disable role

         Args:
            params (RoleDisableRequest): {
                'role_id': 'str',           # required
                'domain_id': 'str',         # injected from auth (required)
            }

        Returns:
            RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        role_vo = self.role_mgr.disable_role_by_vo(role_vo)

        return RoleResponse(**role_vo.to_dict())

    @transaction(permission="identity:Role.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def delete(self, params: RoleDeleteRequest) -> None:
        """delete role

         Args:
            params (RoleDeleteRequest): {
                'role_id': 'str',       # required
                'domain_id': 'str',     # injected from auth (required)
            }

        Returns:
            None
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        if role_vo.is_managed:
            raise ERROR_PERMISSION_DENIED()

        self.role_mgr.delete_role_by_vo(role_vo)

    @transaction(
        permission="identity:Role.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(self, params: RoleGetRequest) -> Union[RoleResponse, dict]:
        """get role

         Args:
            params (RoleGetRequest): {
                'role_id': 'str',           # required
                'domain_id': 'str',         # injected from auth (required)
            }

        Returns:
             RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        return RoleResponse(**role_vo.to_dict())

    @transaction(
        permission="identity:Role.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["role_id", "name", "state", "role_type", "domain_id"])
    @append_keyword_filter(["role_id", "name"])
    @convert_model
    def list(self, params: RoleSearchQueryRequest) -> Union[RolesResponse, dict]:
        """list roles

        Args:
            params (RoleSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'role_id': 'str',
                'name': 'str',
                'state': 'str',
                'role_type': 'str',
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
            RolesResponse:
        """

        query = params.query or {}
        role_vos, total_count = self.role_mgr.list_roles(query, params.domain_id)

        roles_info = [role_vo.to_dict() for role_vo in role_vos]
        return RolesResponse(results=roles_info, total_count=total_count)

    @transaction(
        permission="identity:Role.read",
        role_types=["USER"],
    )
    @append_query_filter(["role_id", "name", "state", "role_type", "domain_id"])
    @append_keyword_filter(["role_id", "name"])
    @convert_model
    def list_basic_role(
        self, params: BasicRoleSearchQueryRequest
    ) -> Union[BasicRolesResponse, dict]:
        """list basic roles

        Args:
            params (RoleSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'role_id': 'str',
                'name': 'str',
                'state': 'str',
                'role_type': 'str',
                'domain_id': 'str',        # injected from auth (required)
            }

        Returns:
            BasicRolesResponse:
        """

        query = params.query or {}
        role_vos, total_count = self.role_mgr.list_roles(query, params.domain_id)

        basic_roles_info = [role_vo.to_dict() for role_vo in role_vos]
        return BasicRolesResponse(results=basic_roles_info, total_count=total_count)

    @transaction(
        permission="identity:Role.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["domain_id"])
    @append_keyword_filter(["role_id", "name"])
    @convert_model
    def stat(self, params: RoleStatQueryRequest) -> dict:
        """stat roles

        Args:
            params (PolicyStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'domain_id': 'str',         # injected from auth (required)
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """

        query = params.query or {}
        return self.role_mgr.stat_roles(query)



from datetime import datetime
from typing import List, Union

from pydantic import BaseModel
from spaceone.core import utils

from spaceone.identity.model.workspace.request import State

__all__ = ["WorkspaceResponse", "WorkspacesResponse"]


class WorkspaceResponse(BaseModel):
    workspace_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    tags: Union[dict, None] = None
    created_by: Union[str, None] = None
    references: Union[list, None] = None
    is_managed: Union[bool, None] = None
    is_dormant: Union[bool, None] = None
    dormant_ttl: Union[int, None] = None
    service_account_count: Union[int, None] = None
    user_count: Union[int, None] = None
    cost_info: Union[dict, None] = None
    trusted_account_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    last_synced_at: Union[datetime, None] = None
    dormant_updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["last_synced_at"] = utils.datetime_to_iso8601(data.get("last_synced_at"))
        data["dormant_updated_at"] = utils.datetime_to_iso8601(
            data.get("dormant_updated_at")
        )
        return data


class WorkspacesResponse(BaseModel):
    results: List[WorkspaceResponse]
    total_count: int



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import agent_pb2, agent_pb2_grpc
from spaceone.identity.service.agent_service import (
    AgentService,
)


class Agent(BaseAPI, agent_pb2_grpc.AgentServicer):
    pb2 = agent_pb2
    pb2_grpc = agent_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.create(params)
        return self.dict_to_message(response)

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.enable(params)
        return self.dict_to_message(response)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.disable(params)
        return self.dict_to_message(response)

    def regenerate(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.regenerate(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        agent_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        agent_svc = AgentService(metadata)
        response: dict = agent_svc.list(params)
        return self.dict_to_message(response)



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import external_auth_pb2, external_auth_pb2_grpc
from spaceone.identity.service.external_auth_service import ExternalAuthService


class ExternalAuth(BaseAPI, external_auth_pb2_grpc.ExternalAuthServicer):
    pb2 = external_auth_pb2
    pb2_grpc = external_auth_pb2_grpc

    def set(self, request, context):
        params, metadata = self.parse_request(request, context)
        external_auth_svc = ExternalAuthService(metadata)
        response: dict = external_auth_svc.set(params)
        return self.dict_to_message(response)

    def unset(self, request, context):
        params, metadata = self.parse_request(request, context)
        external_auth_svc = ExternalAuthService(metadata)
        response: dict = external_auth_svc.unset(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        external_auth_svc = ExternalAuthService(metadata)
        response: dict = external_auth_svc.get(params)
        return self.dict_to_message(response)



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class UserGroup(MongoModel):
    user_group_id = StringField(max_length=40, generate_id="ug", unique=True)
    name = StringField(max_length=255, unique_with=["domain_id", "workspace_id"])
    users = ListField(StringField(max_length=40), default=[])
    tags = DictField(Default=None)
    workspace_id = StringField(max_length=40, required=True)
    domain_id = StringField(max_length=40)

    meta = {
        "updatable_fields": ["name", "users", "tags"],
        "minimal_fields": [
            "user_group_id",
            "name",
            "users",
            "workspace_id",
            "domain_id",
        ],
        "ordering": ["name", "user_group_id"],
        "indexes": [
            "name",
            "workspace_id",
        ],
    }



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *
from spaceone.core.error import *

from spaceone.identity.model.schema.request import *
from spaceone.identity.model.schema.response import *
from spaceone.identity.manager.schema_manager import SchemaManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class SchemaService(BaseService):
    resource = "Schema"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.schema_mgr = SchemaManager()

    @transaction(permission="identity:Schema.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def create(self, params: SchemaCreateRequest) -> Union[SchemaResponse, dict]:
        """create schema

        Args:
            params (SchemaCreateRequest): {
                'schema_id': 'str',         # required
                'name': 'str',              # required
                'schema_type': 'str',       # required
                'schema': 'dict',           # required
                'provider': 'str',          # required
                'related_schemas': 'list',
                'options': 'dict',
                'tags': 'dict',
                'domain_id': 'str'          # injected from auth (required)
            }

        Returns:
            SchemaResponse:
        """

        schema_vo = self.schema_mgr.create_schema(params.dict(by_alias=True))
        return SchemaResponse(**schema_vo.to_dict())

    @transaction(permission="identity:Schema.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def update(self, params: SchemaUpdateRequest) -> Union[SchemaResponse, dict]:
        """update schema

        Args:
            params (SchemaUpdateRequest): {
                'schema_id': 'str',       # required
                'name': 'str',
                'schema': 'dict',
                'related_schemas': 'list',
                'options': 'dict',
                'tags': 'dict',
                'domain_id': 'str'        # injected from auth (required)
            }

        Returns:
            SchemaResponse:
        """

        schema_vo = self.schema_mgr.get_schema(params.schema_id, params.domain_id)
        if schema_vo.is_managed:
            raise ERROR_PERMISSION_DENIED()

        schema_vo = self.schema_mgr.update_schema_by_vo(
            params.dict(exclude_unset=True, by_alias=True), schema_vo
        )

        return SchemaResponse(**schema_vo.to_dict())

    @transaction(permission="identity:Schema.write", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def delete(self, params: SchemaDeleteRequest) -> None:
        """delete schema

        Args:
            params (SchemaDeleteRequest): {
                'schema_id': 'str',     # required
                'domain_id': 'str'      # injected from auth (required)
            }

        Returns:
            None
        """

        schema_vo = self.schema_mgr.get_schema(params.schema_id, params.domain_id)
        if schema_vo.is_managed:
            raise ERROR_PERMISSION_DENIED()

        self.schema_mgr.delete_schema_by_vo(schema_vo)

    @transaction(
        permission="identity:Schema.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(self, params: SchemaGetRequest) -> Union[SchemaResponse, dict]:
        """delete schema

        Args:
            params (SchemaGetRequest): {
                'schema_id': 'str',     # required
                'domain_id': 'str'      # injected from auth (required)
            }

        Returns:
            SchemaResponse:
        """

        schema_vo = self.schema_mgr.get_schema(params.schema_id, params.domain_id)
        return SchemaResponse(**schema_vo.to_dict())

    @transaction(
        permission="identity:Schema.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(
        [
            "schema_id",
            "name",
            "schema_type",
            "provider",
            "related_schema_id",
            "is_managed",
            "domain_id",
        ]
    )
    @append_keyword_filter(["schema_id", "name"])
    @convert_model
    def list(self, params: SchemaSearchQueryRequest) -> Union[SchemasResponse, dict]:
        """list schemas

        Args:
            params (SchemaSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'schema_id': 'str',
                'name': 'str',
                'schema_type': 'str',
                'provider': 'str',
                'related_schema_id': 'str',
                'is_managed': 'bool',
                'domain_id': 'str'              # injected from auth (required)
            }

        Returns:
            SchemasResponse:
        """

        query = params.query or {}
        schema_vos, total_count = self.schema_mgr.list_schemas(query, params.domain_id)

        schemas_info = [schema_vo.to_dict() for schema_vo in schema_vos]
        return SchemasResponse(results=schemas_info, total_count=total_count)

    @transaction(
        permission="identity:Schema.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["domain_id"])
    @append_keyword_filter(["schema_id", "name"])
    @convert_model
    def stat(self, params: SchemaStatQueryRequest) -> dict:
        """stat schemas

        Args:
            params (SchemaStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'domain_id': 'str'    # injected from auth (required)
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }

        """

        query = params.query or {}
        return self.schema_mgr.stat_schemas(query)



import logging
from typing import Tuple

from mongoengine import QuerySet
from spaceone.core.manager import BaseManager

from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.model.workspace_group.database import WorkspaceGroup

_LOGGER = logging.getLogger(__name__)


class WorkspaceGroupManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.workspace_group_model = WorkspaceGroup
        self.rb_mgr = RoleBindingManager()

    def create_workspace_group(self, params: dict) -> WorkspaceGroup:
        def _rollback(vo: WorkspaceGroup):
            _LOGGER.info(
                f"[create_workspace_group._rollback] Delete workspace group : {vo.workspace_group_id} ({vo.name})"
            )
            vo.delete()

        workspace_group_vo = self.workspace_group_model.create(params)
        self.transaction.add_rollback(_rollback, workspace_group_vo)

        return workspace_group_vo

    def update_workspace_group_by_vo(
        self, params: dict, workspace_group_vo: WorkspaceGroup
    ) -> WorkspaceGroup:
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_workspace_group._rollback] Revert Data : {old_data['workspace_group_id']} ({old_data['name']})"
            )
            workspace_group_vo.update(old_data)

        self.transaction.add_rollback(_rollback, workspace_group_vo.to_dict())

        return workspace_group_vo.update(params)

    def delete_workspace_group_by_vo(self, workspace_group_vo: WorkspaceGroup) -> None:
        user_ids = [user["user_id"] for user in workspace_group_vo.users]
        rb_vos = self.rb_mgr.filter_role_bindings(
            user_id=user_ids,
            workspace_group_id=workspace_group_vo.workspace_group_id,
            domain_id=workspace_group_vo.domain_id,
        )

        if rb_vos.count() > 0:
            _LOGGER.debug(
                f"[delete_workspace_group_by_vo] Delete role bindings count with {workspace_group_vo.workspaces}: {rb_vos.count()}"
            )
            rb_vos.delete()

        workspace_group_vo.delete()

    # TODO: When add_users and remove_users, are user_id and role_type required?
    def get_workspace_group(
        self,
        workspace_group_id: str,
        domain_id: str,
        user_id: str = None,
        role_type: str = None,
    ) -> WorkspaceGroup:
        conditions = {
            "workspace_group_id": workspace_group_id,
            "domain_id": domain_id,
        }

        if user_id:
            conditions["users__user_id"] = user_id

        # TODO: Check if this is correct
        # if role_type:
        #     conditions["users__role_type"] = role_type

        return self.workspace_group_model.get(**conditions)

    def filter_workspace_groups(self, **conditions) -> QuerySet:
        return self.workspace_group_model.filter(**conditions)

    def list_workspace_groups(self, query: dict) -> Tuple[QuerySet, int]:
        return self.workspace_group_model.query(**query)

    def stat_workspace_group(self, query: dict) -> dict:
        return self.workspace_group_model.stat(**query)

    @staticmethod
    def check_user_id_in_users(user_id: str, workspace_group_vo: WorkspaceGroup):
        return any(
            workspace_group_user_id == user_id
            for workspace_group_user_id in workspace_group_vo["users"]
        )



import logging
from datetime import datetime
from typing import Union

from spaceone.core.error import (
    ERROR_INVALID_PARAMETER,
    ERROR_NOT_FOUND,
    ERROR_PERMISSION_DENIED,
)
from spaceone.core.service import (
    BaseService,
    authentication_handler,
    authorization_handler,
    event_handler,
    mutation_handler,
    transaction,
)
from spaceone.core.service.utils import (
    append_keyword_filter,
    append_query_filter,
    convert_model,
)

from spaceone.identity.error import ERROR_ROLE_DOES_NOT_EXIST_OF_USER
from spaceone.identity.error.error_role import (
    ERROR_NOT_ALLOWED_ROLE_TYPE,
    ERROR_NOT_ALLOWED_USER_STATE,
)
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.workspace_group_manager import WorkspaceGroupManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.manager.workspace_user_manager import WorkspaceUserManager
from spaceone.identity.model.workspace_group.request import (
    WorkspaceGroupAddUsersRequest,
    WorkspaceGroupAddWorkspacesRequest,
    WorkspaceGroupCreateRequest,
    WorkspaceGroupDeleteRequest,
    WorkspaceGroupGetRequest,
    WorkspaceGroupRemoveUsersRequest,
    WorkspaceGroupRemoveWorkspacesRequest,
    WorkspaceGroupSearchQueryRequest,
    WorkspaceGroupStatQueryRequest,
    WorkspaceGroupUpdateRequest,
    WorkspaceGroupUpdateRoleRequest,
)
from spaceone.identity.model.workspace_group.response import (
    WorkspaceGroupResponse,
    WorkspaceGroupsResponse,
)
from spaceone.identity.service.role_binding_service import RoleBindingService

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class WorkspaceGroupService(BaseService):
    resource = "WorkspaceGroup"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.workspace_group_mgr = WorkspaceGroupManager()
        self.workspace_user_mgr = WorkspaceUserManager()
        self.user_mgr = UserManager()
        self.role_mgr = RoleManager()
        self.rb_svc = RoleBindingService()
        self.rb_mgr = RoleBindingManager()

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def create(
        self, params: WorkspaceGroupCreateRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Create workspace group
        Args:
            params (WorkspaceGroupCreateRequest): {
                'name': 'str',                  # required
                'tags': 'dict',
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """
        params_data = params.dict()
        params_data["created_by"] = self.transaction.get_meta("authorization.user_id")

        workspace_group_vo = self.workspace_group_mgr.create_workspace_group(
            params_data
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def update(
        self, params: WorkspaceGroupUpdateRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Update workspace group
        Args:
            params (WorkspaceGroupUpdateRequest): {
                'workspace_group_id': 'str',    # required
                'name': 'str',
                'tags': 'dict',
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str',             # inje     WorkspaceGroupResponse:
        """
        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        params_data = params.dict()
        params_data["updated_by"] = self.transaction.get_meta("authorization.user_id")
        params_data["updated_at"] = datetime.utcnow()

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params_data, workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def delete(self, params: WorkspaceGroupDeleteRequest) -> None:
        """Delete workspace group
        Args:
            params (WorkspaceGroupDeleteRequest): {
                'workspace_group_id': 'str',    # required
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }
        Returns:
            None
        """
        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        self.workspace_group_mgr.delete_workspace_group_by_vo(workspace_group_vo)

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def add_workspaces(
        self, params: WorkspaceGroupAddWorkspacesRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Add workspaces to workspace group
        Args:
            params (WorkspaceGroupAddWorkspacesRequest): {
                'workspace_group_id': 'str',           # required
                'workspaces': 'List[str]',             # required
                'workspace_id': 'str',                 # injected from auth
                'domain_id': 'str',                    # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """
        workspace_mgr = WorkspaceManager()
        query = {"filter": [{"k": "domain_id", "v": params.domain_id, "o": "eq"}]}
        workspace_list, count = workspace_mgr.list_workspaces(query)
        workspace_id_list = [workspace.workspace_id for workspace in workspace_list]

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        all_workspaces_exist = all(
            params_workspace_id in workspace_id_list
            for params_workspace_id in params.workspaces
        )

        users = workspace_group_vo.users or []
        user_role_ids = list(set([user_info["role_id"] for user_info in users]))
        role_vos = self.role_mgr.filter_roles(
            role_id=user_role_ids, domain_id=params.domain_id
        )
        role_id_map = {
            user_info["user_id"]: user_info["role_id"] for user_info in users
        }

        if len(user_role_ids) != len(role_vos):
            for user_info in users:
                if user_info["user_id"] not in role_id_map:
                    raise ERROR_ROLE_DOES_NOT_EXIST_OF_USER(
                        role_id=user_info["role_id"], user_id=user_info["user_id"]
                    )

        if not all_workspaces_exist:
            raise ERROR_NOT_FOUND(key="workspaces", value=params.workspaces)
        elif (len(params.workspaces) > 0) and all_workspaces_exist:
            workspaces = workspace_group_vo.workspaces or []
            workspaces.extend(params.workspaces)
            params.workspaces = list(set(workspaces))

            role_id_map = {
                user_info["user_id"]: user_info["role_id"] for user_info in users
            }

            workspaces = workspace_group_vo.workspaces or []
            for user_info in users:
                for workspace_id in workspaces:
                    role_binding_params = {
                        "user_id": user_info["user_id"],
                        "role_id": role_id_map[user_info["user_id"]],
                        "resource_group": "WORKSPACE",
                        "domain_id": params.domain_id,
                        "workspace_group_id": params.workspace_group_id,
                        "workspace_id": workspace_id,
                    }
                    self.rb_svc.create_role_binding(role_binding_params)

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def remove_workspaces(
        self, params: WorkspaceGroupRemoveWorkspacesRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Remove workspaces from workspace group
        Args:
            params (WorkspaceGroupRemoveWorkspacesRequest): {
                'workspace_group_id': 'str',              # required
                'workspaces': 'List[str]',                # required
                'workspace_id': 'str',                    # injected from auth
                'domain_id': 'str',                       # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """
        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        workspaces = workspace_group_vo.workspaces or []
        params.workspaces = list(set(workspaces) - set(params.workspaces))

        rb_vos = self.rb_mgr.filter_role_bindings(
            workspace_id=params.workspaces,
            workspace_group_id=workspace_group_vo.workspace_group_id,
            domain_id=params.domain_id,
        )
        _LOGGER.info(
            f"[remove_workspaces] deleted role bindings count: {rb_vos.count()}"
        )
        rb_vos.delete()

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def add_users(
        self, params: WorkspaceGroupAddUsersRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Add users to workspace group
        Args:
            params (WorkspaceGroupAddUsersRequest): {
                'workspace_group_id': 'str',      # required
                'users': [
                    {
                        'user_id': 'str',
                        'role_id': 'str',
                    }
                ],                                # required
                'workspace_id': 'str',
                'domain_id': 'str',               # injected from auth (required)
            }
        Returns:
           WorkspaceGroupResponse:
        """
        owner_type = self.transaction.get_meta("authorization.owner_type")
        role_type = self.transaction.get_meta("authorization.role_type")

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        new_users = list(set([user_info["user_id"] for user_info in params.users]))
        old_users = []
        if workspace_group_vo.users:
            old_users = list(
                set([user_info["user_id"] for user_info in workspace_group_vo.users])
            )

        if set(new_users) & set(old_users):
            _LOGGER.error(
                f"Users {new_users} is already added to the workspace group or not registered."
            )
            raise ERROR_INVALID_PARAMETER(
                key="users",
                reason=f"User {new_users} is already added to the workspace group or not registered.",
            )

        user_vos = self.user_mgr.filter_users(
            domain_id=params.domain_id, user_id=new_users
        )

        if not user_vos.count() == len(new_users):
            raise ERROR_NOT_FOUND(key="user_id", value=new_users)

        if role_type == "WORKSPACE_OWNER":
            if owner_type == "APP":
                _LOGGER.error("Workspace Owner role type by app cannot add users.")
                raise ERROR_PERMISSION_DENIED()
            else:
                user_id = self.transaction.get_meta("authorization.user_id")
                if user_id not in old_users:
                    _LOGGER.error(f"User ID {user_id} is not in workspace group.")
                    raise ERROR_PERMISSION_DENIED()
        elif role_type != "DOMAIN_ADMIN":
            raise ERROR_PERMISSION_DENIED()

        users = params.users or []
        workspaces = workspace_group_vo.workspaces or []

        role_ids = list(set([user["role_id"] for user in users]))
        role_vos = self.role_mgr.filter_roles(
            role_id=role_ids,
            domain_id=params.domain_id,
            role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
        )

        role_map = {role_vo.role_id: role_vo.role_type for role_vo in role_vos}

        user_info = {user_vo["user_id"]: user_vo for user_vo in user_vos}
        for user in users:
            user_id = user["user_id"]
            if user_id in user_info:
                user["user_name"] = user_info[user_id]["name"]
                user["state"] = user_info[user_id]["state"]

        new_users = []
        for user in users:
            role_type = role_map[user["role_id"]]

            for workspace_id in workspaces:
                role_binding_params = {
                    "user_id": user["user_id"],
                    "role_id": user["role_id"],
                    "role_type": role_type,
                    "resource_group": "WORKSPACE",
                    "domain_id": params.domain_id,
                    "workspace_group_id": params.workspace_group_id,
                    "workspace_id": workspace_id,
                }
                self.rb_svc.create_role_binding(role_binding_params)

            new_users.append(
                {
                    "user_id": user["user_id"],
                    "role_id": user["role_id"],
                    "role_type": role_type,
                    "user_name": user["user_name"],
                    "state": user["state"],
                }
            )

        old_users = workspace_group_vo.users or []

        params.users = old_users + new_users

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def remove_users(
        self, params: WorkspaceGroupRemoveUsersRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Remove users from workspace group
        Args:
            params (WorkspaceGroupRemoveUsersRequest): {
                'workspace_group_id': 'str',         # required
                'users': [
                    'user_id': 'str'
                ],                                   # required
                'workspace_id': 'str',
                'domain_id': 'str',                  # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """
        role_type = self.transaction.get_meta("authorization.role_type")
        owner_type = self.transaction.get_meta("authorization.owner_type")

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )

        if role_type == "WORKSPACE_OWNER":
            if owner_type == "APP":
                _LOGGER.error("Workspace Owner role type by app cannot remove users.")
                raise ERROR_PERMISSION_DENIED()
            else:
                user_id = self.transaction.get_meta("authorization.user_id")
                workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
                    params.workspace_group_id,
                    params.domain_id,
                    user_id=user_id,
                )
                if workspace_group_vo:
                    if not self.workspace_group_mgr.check_user_id_in_users(
                        user_id, workspace_group_vo
                    ):
                        _LOGGER.error(f"User ID {user_id} is not in workspace group.")
                        ERROR_NOT_FOUND(
                            key="workspace_group_id", value=params.workspace_group_id
                        )
                else:
                    ERROR_NOT_FOUND(
                        key="workspace_group_id", value=params.workspace_group_id
                    )
        elif role_type != "DOMAIN_ADMIN":
            raise ERROR_PERMISSION_DENIED()

        workspace_group_user_ids = [
            user["user_id"] for user in workspace_group_vo.users
        ]
        params_user_ids = list(set([user["user_id"] for user in params.users]))

        for params_user_id in params_user_ids:
            if params_user_id not in workspace_group_user_ids:
                raise ERROR_NOT_FOUND(key="params_user_id", value=params_user_id)

        workspace_group_users = [users for users in workspace_group_vo["users"]]
        rb_vos = self.rb_mgr.filter_role_bindings(
            user_id=params_user_ids,
            workspace_group_id=params.workspace_group_id,
            domain_id=params.domain_id,
        )

        if rb_vos.count() > 0:
            _LOGGER.debug(
                f"[remove_users] Delete role bindings count with {workspace_group_vo.workspaces}: {rb_vos.count()}"
            )
            rb_vos.delete()

        params.users = []
        for user in workspace_group_users:
            if user["user_id"] not in params_user_ids:
                params.users.append(user)

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            params.dict(exclude_unset=True), workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(
        permission="identity:WorkspaceGroup.write", role_types=["DOMAIN_ADMIN"]
    )
    @convert_model
    def update_role(
        self, params: WorkspaceGroupUpdateRoleRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Update workspace group's user's role

        Args:
            params (WorkspaceGroupUpdateRoleRequest): {
                'workspace_group_id': 'str',        # required
                'user_id': 'str',                   # required
                'role_id': 'str',                   # required
                'workspace_id': 'str',              # injected from auth (required)
                'domain_id': 'str',                 # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """

        workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
            params.workspace_group_id, params.domain_id
        )
        user_vo = self.user_mgr.get_user(params.user_id, params.domain_id)

        if user_vo.state in ["DISABLED", "DELETED"]:
            _LOGGER.error(f"User ID {user_vo.user_id}'s state is {user_vo.state}.")
            raise ERROR_NOT_ALLOWED_USER_STATE(
                user_id=user_vo.user_id, state=user_vo.state
            )

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        if role_vo.role_type not in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            raise ERROR_NOT_ALLOWED_ROLE_TYPE()

        role_binding_vos = self.rb_mgr.filter_role_bindings(
            user_id=user_vo.user_id,
            workspace_group_id=workspace_group_vo.workspace_group_id,
            domain_id=params.domain_id,
        )

        for role_binding_vo in role_binding_vos:
            role_binding_vo.update(
                {"role_id": params.role_id, "role_type": role_vo.role_type}
            )

        update_workspace_group_params = {"users": workspace_group_vo.users or []}
        for user_info in update_workspace_group_params.get("users", []):
            if user_info["user_id"] == user_vo.user_id:
                user_info["role_id"] = params.role_id
                user_info["role_type"] = role_vo.role_type
                user_info["user_name"] = user_vo.name
                user_info["state"] = user_vo.state
                break

        workspace_group_vo = self.workspace_group_mgr.update_workspace_group_by_vo(
            update_workspace_group_params, workspace_group_vo
        )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @convert_model
    @transaction(permission="identity:WorkspaceGroup.read", role_types=["DOMAIN_ADMIN"])
    @convert_model
    def get(
        self, params: WorkspaceGroupGetRequest
    ) -> Union[WorkspaceGroupResponse, dict]:
        """Get workspace group

        Args:
            params (WorkspaceGroupGetRequest): {
                'workspace_group_id': 'str', # required
                'workspace_id': 'str',
                'domain_id': 'str',          # injected from auth (required)
            }
        Returns:
            WorkspaceGroupResponse:
        """
        user_id = self.transaction.get_meta("authorization.user_id")
        role_type = self.transaction.get_meta("authorization.role_type")

        workspace_group_vo = None
        if role_type == "DOMAIN_ADMIN":
            workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
                params.workspace_group_id, params.domain_id
            )
        elif role_type in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            workspace_group_vo = self.workspace_group_mgr.get_workspace_group(
                params.workspace_group_id, params.domain_id, user_id
            )

        return WorkspaceGroupResponse(**workspace_group_vo.to_dict())

    @transaction(permission="identity:WorkspaceGroup.read", role_types=["DOMAIN_ADMIN"])
    @append_query_filter(["workspace_group_id", "name", "domain_id"])
    @append_keyword_filter(["workspace_group_id", "name"])
    @convert_model
    def list(
        self, params: WorkspaceGroupSearchQueryRequest
    ) -> Union[WorkspaceGroupsResponse, dict]:
        """List workspace groups
        Args:
            params (WorkspaceGroupSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'workspace_group_id': 'str',         # required
                'name': 'str',
                'created_by': 'str',
                'updated_by': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',                  # injected from auth (required)
            }
        Returns:
            WorkspaceGroupsResponse:
        """
        user_id = self.transaction.get_meta("authorization.user_id")
        role_type = self.transaction.get_meta("authorization.role_type")

        query = {}
        if role_type == "DOMAIN_ADMIN":
            query = params.query
        elif role_type in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            query = params.query
            query["filter"] = query.get("filter", [])
            query["filter"].append(
                {
                    "k": "users",
                    "v": user_id,
                    "o": "contain",
                }
            )

        workspace_group_vos, total_count = (
            self.workspace_group_mgr.list_workspace_groups(query)
        )

        workspace_groups_info = [
            workspace_group_vo.to_dict() for workspace_group_vo in workspace_group_vos
        ]

        return WorkspaceGroupsResponse(
            results=workspace_groups_info, total_count=total_count
        )

    @transaction(permission="identity:WorkspaceGroup.read", role_types=["DOMAIN_ADMIN"])
    @append_query_filter(["workspace_id", "workspace_group_id", "domain_id"])
    @convert_model
    def stat(self, params: WorkspaceGroupStatQueryRequest) -> dict:
        """Stat workspace groups
        Args:
            params (WorkspaceGroupStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',             # injected from auth (required)
                'domain_id': 'str',                # injected from auth (required)
            }
        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """
        user_id = self.transaction.get_meta("authorization.user_id")
        role_type = self.transaction.get_meta("authorization.role_type")

        query = {}
        if role_type == "DOMAIN_ADMIN":
            query = params.query
        elif role_type in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            query = params.query
            query["filter"] = query.get("filter", [])
            query["filter"].append(
                {
                    "k": "users",
                    "v": user_id,
                    "o": "contain",
                }
            )

        if params.workspace_id:
            query = params.query
        return self.workspace_group_mgr.stat_workspace_group(query)



from datetime import datetime
from typing import List, Union

from pydantic import BaseModel
from spaceone.core import utils

from spaceone.identity.model.role_binding.request import ResourceGroup

__all__ = ["RoleBindingResponse", "RoleBindingsResponse"]


class RoleBindingResponse(BaseModel):
    role_binding_id: Union[str, None] = None
    user_id: Union[str, None] = None
    role_id: Union[str, None] = None
    role_type: Union[str, None] = None
    resource_group: Union[ResourceGroup, None] = None
    workspace_id: Union[str, None] = None
    workspace_group_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        return data


class RoleBindingsResponse(BaseModel):
    results: List[RoleBindingResponse]
    total_count: int



from typing import Dict, List, Literal, Union

from pydantic import BaseModel

__all__ = [
    "WorkspaceGroupUserAddRequest",
    "WorkspaceGroupUserRemoveRequest",
    "WorkspaceGroupUserUpdateRoleRequest",
    "WorkspaceGroupUserFindRequest",
    "WorkspaceGroupUserGetRequest",
    "WorkspaceGroupUserSearchQueryRequest",
    "WorkspaceGroupUserStatQueryRequest",
]

State = Literal["ENABLED", "DISABLED", "PENDING"]


class WorkspaceGroupUserAddRequest(BaseModel):
    workspace_group_id: str
    users: List[Dict[str, str]]
    user_id: str
    domain_id: str


class WorkspaceGroupUserRemoveRequest(BaseModel):
    workspace_group_id: str
    users: List[Dict[str, str]]
    user_id: str
    domain_id: str


class WorkspaceGroupUserUpdateRoleRequest(BaseModel):
    workspace_group_id: str
    role_id: str
    target_user_id: str
    user_id: str
    domain_id: str


class WorkspaceGroupUserFindRequest(BaseModel):
    workspace_group_id: str
    keyword: Union[str, None] = None
    state: Union[State, None] = None
    page: Union[dict, None] = None
    user_id: str
    domain_id: str


class WorkspaceGroupUserGetRequest(BaseModel):
    workspace_group_id: str
    user_id: str
    domain_id: str


class WorkspaceGroupUserSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    workspace_group_id: Union[str, None] = None
    name: Union[str, None] = None
    created_by: Union[str, None] = None
    updated_by: Union[str, None] = None
    user_id: str
    domain_id: str


class WorkspaceGroupUserStatQueryRequest(BaseModel):
    query: dict
    workspace_group_id: str
    user_id: str
    domain_id: str



from spaceone.core.error import *


class ERROR_GENERATE_KEY_FAILURE(ERROR_BASE):
    _message = "Error on generate key."


class ERROR_MANAGED_RESOURCE_CAN_NOT_BE_MODIFIED(ERROR_UNKNOWN):
    message = "Managed resource can not be deleted. please disable schedule first."


class ERROR_WORKSPACES_DO_NOT_EXIST(ERROR_UNKNOWN):
    _message = "Resource Not Found. (key = {key}, reason = {reason})"


class ERROR_ROLE_DOES_NOT_EXIST_OF_USER(ERROR_NOT_FOUND):
    _message = "Role does not exist of User. (role_id = {role_id}, user_id = {user_id})"



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.plugin import (
    external_auth_pb2,
    external_auth_pb2_grpc,
)
from spaceone.identity.plugin.external_auth.service.external_auth_service import (
    ExternalAuthService,
)


class ExternalAuth(BaseAPI, external_auth_pb2_grpc.ExternalAuthServicer):
    pb2 = external_auth_pb2
    pb2_grpc = external_auth_pb2_grpc

    def init(self, request, context):
        params, metadata = self.parse_request(request, context)
        external_auth_svc = ExternalAuthService(metadata)
        response: dict = external_auth_svc.init(params)
        return self.dict_to_message(response)

    def authorize(self, request, context):
        params, metadata = self.parse_request(request, context)
        external_auth_svc = ExternalAuthService(metadata)
        response: dict = external_auth_svc.authorize(params)
        return self.dict_to_message(response)



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.identity.plugin.account_collector.interface.grpc.account_collector import (
    AccountCollector,
)

_all_ = ["app"]

app = GRPCServer()
app.add_service(AccountCollector)



from mongoengine import StringField, DictField, DateTimeField
from spaceone.core.model.mongo_model import MongoModel


class Agent(MongoModel):
    agent_id = StringField(max_length=40, generate_id="agent", unique=True)
    options = DictField(default=None)
    app_id = StringField(max_length=40)
    service_account_id = StringField(max_length=40)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "minimal_fields": [
            "agent_id",
            "options",
            "service_account_id",
        ],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["-created_at"],
        "indexes": [
            "service_account_id",
            "workspace_id",
            "domain_id",
        ],
    }



from spaceone.identity.model.agent.database import Agent
from spaceone.identity.model.app.database import App
from spaceone.identity.model.domain.database import Domain
from spaceone.identity.model.external_auth.database import ExternalAuth
from spaceone.identity.model.job.database import Job
from spaceone.identity.model.project.database import Project
from spaceone.identity.model.project_group.database import ProjectGroup
from spaceone.identity.model.provider.database import Provider
from spaceone.identity.model.role.database import Role
from spaceone.identity.model.role_binding.database import RoleBinding
from spaceone.identity.model.schema.database import Schema
from spaceone.identity.model.service_account.database import ServiceAccount
from spaceone.identity.model.trusted_account.database import TrustedAccount
from spaceone.identity.model.user.database import User
from spaceone.identity.model.user_group.database import UserGroup
from spaceone.identity.model.workspace.database import Workspace
from spaceone.identity.model.workspace_group.database import WorkspaceGroup



from datetime import datetime
from typing import List, Literal, Union

from pydantic import BaseModel
from spaceone.core import utils

__all__ = ["MyWorkspaceResponse", "MyWorkspacesResponse"]

from spaceone.identity.model.role_binding.response import RoleBindingResponse

State = Literal["ENABLED", "DISABLED"]


class MyWorkspaceResponse(BaseModel):
    workspace_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    role_name: Union[str, None] = None
    role_type: Union[str, None] = None
    tags: Union[dict, None] = None
    created_by: Union[str, None] = None
    reference_id: Union[str, None] = None
    is_managed: Union[bool, None] = None
    is_dormant: Union[bool, None] = None
    role_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    last_synced_at: Union[datetime, None] = None
    dormant_updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["last_synced_at"] = utils.datetime_to_iso8601(data.get("last_synced_at"))
        data["dormant_updated_at"] = utils.datetime_to_iso8601(
            data.get("dormant_updated_at")
        )
        return data


class MyWorkspacesResponse(BaseModel):
    results: List[MyWorkspaceResponse]


class MyWorkspaceGroupResponse(BaseModel):
    workspace_group_id: Union[str, None] = None
    name: Union[str, None] = None
    workspaces: Union[List[str], None] = None
    users: Union[List[dict], None] = None
    tags: Union[dict, None] = None
    role_binding_info: Union[RoleBindingResponse, None] = None
    created_by: Union[str, None] = None
    updated_by: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["updated_at"] = utils.datetime_to_iso8601(data["updated_at"])
        return data


class MyWorkspaceGroupsResponse(BaseModel):
    results: List[MyWorkspaceGroupResponse]
    total_count: int



from typing import Union, Literal
from pydantic import BaseModel

__all__: ["TokenIssueRequest"]

AuthType = Literal["LOCAL", "EXTERNAL", "MFA"]
GrantType = Literal["REFRESH_TOKEN"]
Scope = Literal["SYSTEM", "DOMAIN", "WORKSPACE", "USER"]


class TokenIssueRequest(BaseModel):
    credentials: dict
    auth_type: AuthType
    timeout: Union[int, None] = None
    verify_code: Union[str, None] = None
    domain_id: str


class TokenGrantRequest(BaseModel):
    grant_type: str
    token: str
    scope: Scope
    timeout: Union[int, None] = None
    permissions: Union[list, None] = None
    workspace_id: Union[str, None] = None
    domain_id: Union[str, None] = None



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.core.v2 import handler_pb2
from google.protobuf.json_format import ParseDict
from spaceone.api.identity.v2 import domain_pb2, domain_pb2_grpc
from spaceone.identity.service.domain_service import DomainService


class Domain(BaseAPI, domain_pb2_grpc.DomainServicer):
    pb2 = domain_pb2
    pb2_grpc = domain_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.update(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        domain_svc.delete(params)
        return self.empty()

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.enable(params)
        return self.dict_to_message(response)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.disable(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.get(params)
        return self.dict_to_message(response)

    def get_auth_info(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.get_auth_info(params)
        return self.dict_to_message(response)

    def get_public_key(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.get_public_key(params)
        return ParseDict(response, handler_pb2.AuthenticationResponse())

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        domain_svc = DomainService(metadata)
        response: dict = domain_svc.stat(params)
        return self.dict_to_message(response)



import logging
from typing import Tuple, List, Union

from spaceone.core.manager import BaseManager
from spaceone.identity.model.trusted_account.database import TrustedAccount

_LOGGER = logging.getLogger(__name__)


class TrustedAccountManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_account_model = TrustedAccount

    def create_trusted_account(self, params: dict) -> TrustedAccount:
        def _rollback(vo: TrustedAccount):
            _LOGGER.info(
                f"[create_trusted_account._rollback] "
                f"Delete trusted service account: {vo.name} ({vo.trusted_account_id})"
            )
            vo.delete()

        trusted_account_vo = self.trusted_account_model.create(params)
        self.transaction.add_rollback(_rollback, trusted_account_vo)

        return trusted_account_vo

    def update_trusted_account_by_vo(
        self, params: dict, trusted_account_vo: TrustedAccount
    ) -> TrustedAccount:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_trusted_account_by_vo._rollback] Revert Data : "
                f'{old_data["trusted_account_id"]}'
            )
            trusted_account_vo.update(old_data)

        self.transaction.add_rollback(_rollback, trusted_account_vo.to_dict())

        return trusted_account_vo.update(params)

    @staticmethod
    def delete_trusted_account_by_vo(trusted_account_vo: TrustedAccount) -> None:
        trusted_account_vo.delete()

    def get_trusted_account(
        self,
        trusted_account_id: str,
        domain_id: str,
        workspace_id: Union[str, list] = None,
    ) -> TrustedAccount:
        conditions = {
            "trusted_account_id": trusted_account_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        return self.trusted_account_model.get(**conditions)

    def filter_trusted_accounts(self, **conditions) -> List[TrustedAccount]:
        return self.trusted_account_model.filter(**conditions)

    def list_trusted_accounts(self, query: dict) -> Tuple[list, int]:
        return self.trusted_account_model.query(**query)

    def stat_trusted_accounts(self, query: dict) -> dict:
        return self.trusted_account_model.stat(**query)



from typing import Union, List
from pydantic import BaseModel, Field

__all__ = ["PluginResponse", "AccountsResponse"]


class PluginResponse(BaseModel):
    metadata: dict


class AccountResponse(BaseModel):
    name: Union[str, None] = None
    resource_id: Union[str, None] = None
    data: Union[dict, None] = None
    secret_schema_id: Union[str, None] = None
    secret_data: Union[dict, None] = None
    tags: Union[dict, None] = None
    location: Union[list, None] = None


class AccountsResponse(BaseModel):
    results: List[AccountResponse] = None



from typing import Literal, Union, List
from pydantic import BaseModel

__all__ = ["TokenResponse", "GrantTokenResponse"]

RoleType = Literal["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER", "USER"]


class TokenResponse(BaseModel):
    access_token: str
    refresh_token: str


class GrantTokenResponse(BaseModel):
    access_token: str
    role_type: RoleType
    role_id: Union[str, None] = None
    page_access: Union[List[str], None] = None
    domain_id: str
    workspace_id: Union[str, None] = None
